from NuRadioReco.modules.base.module import register_run
from NuRadioReco.utilities import units
from NuRadioReco.modules.trigger.highLowThreshold import get_majority_logic
from NuRadioReco.framework.trigger import EnvelopeTrigger
import NuRadioReco.utilities.fft
import numpy as np
import scipy.signal
import copy
import time
import logging

logger = logging.getLogger('envelopeTrigger')


def get_envelope_triggers(trace, threshold):  # define trigger constraint for each channel

    """
    calculates a Hilbert-envelope based trigger

    Parameters
    ----------
    trace: array of floats
        the signal trace
    threshold: float
        the threshold
    Returns
    -------
    triggered bins: array of bools
        the bins where the trigger condition is satisfied
    """

    return np.abs(scipy.signal.hilbert(trace)) > threshold


class triggerSimulator:
    """
    Calculate a simple amplitude trigger depending on the Hilbert-envelope.
    """

    def __init__(self):
        self.__t = 0
        self.begin()

    def begin(self, debug=False):
        self.__debug = debug


    @register_run()
    def run(self, evt, station, det, passband, order, threshold, coinc_window, number_coincidences=2, triggered_channels=None,
             trigger_name='envelope_trigger'):
        """
        Simulates simple threshold trigger based on an Hilbert-envelope of the trace. Passband of the trigger, coincidence
        window within different channels should have triggered, and the number of channels needed to trigger can be specified.

        Parameters
        ----------


        threshold: float
            threshold above (or below) a trigger is issued, absolute amplitude
        number_coincidences: int
            number of channels that are required in coincidence to trigger a station
        triggered_channels: array of ints or None
            channels ids that are triggered on, if None trigger will run on all channels
        coinc_window: float
            time window in which number_coincidences channels need to trigger
        trigger_name: string
            a unique name of this particular trigger
        """


        t = time.time()  # absolute time of system

        sampling_rate = station.get_channel(0).get_sampling_rate()
        dt = 1. / sampling_rate

        triggered_bins_channels = []
        channels_that_passed_trigger = []

        if triggered_channels is None:  # caveat: all channels start at the same time
            for channel in station.iter_channels():
                channel_trace_start_time = channel.get_trace_start_time()
                break
        else:
            channel_trace_start_time = station.get_channel(triggered_channels[0]).get_trace_start_time()

        event_id = evt.get_id()
        for channel in station.iter_channels():
            # get filter
            frequencies = channel.get_frequencies()

            f = np.zeros_like(frequencies, dtype=np.complex)
            mask = frequencies > 0
            b, a = scipy.signal.butter(order, passband, 'bandpass', analog=True)  #Numerator (b) and denominator (a) polynomials of the IIR filter
            w, h = scipy.signal.freqs(b, a, frequencies[mask])  #	w :The angular frequencies at which h was computed. h :The frequency response.
            f[mask] = h

            # apply filter
            freq_spectrum_fft = channel.get_frequency_spectrum()
            freq_spectrum_fft_copy = copy.copy(freq_spectrum_fft)  #copy spectrum so it is only changed within the trigger module
            sampling_rate = channel.get_sampling_rate()

            freq_spectrum_fft_copy *= f
            trace_filtered = NuRadioReco.utilities.fft.freq2time(freq_spectrum_fft_copy, sampling_rate)

            # apply envelope trigger to each channel
            channel_id = channel.get_id()

            trace = trace_filtered
            if triggered_channels is not None and channel_id not in triggered_channels:
                logger.debug("skipping channel {}".format(channel_id))
                continue
            if channel.get_trace_start_time() != channel_trace_start_time:
                logger.warning('Channel has a trace_start_time that differs from '
                               '        the other channels. The trigger simulator may not work properly')

            triggered_bins = get_envelope_triggers(trace, threshold)
            triggered_bins_channels.append(triggered_bins)

            if True in triggered_bins:
                channels_that_passed_trigger.append(channel.get_id())

        # check for coincidences with get_majority_logic(tts, number_of_coincidences=2,
        # time_coincidence=32 * units.ns, dt=1 * units.ns)
        # returns:
        # triggered: bool; returns True if majority logic is fulfilled --> has_triggered
        # triggered_bins: array of ints; the bins that fulfilled the trigger --> triggered_bins
        # triggered_times = triggered_bins * dt: array of floats;
        # the trigger times relative to the trace --> triggered_times

        has_triggered, triggered_bins, triggered_times = get_majority_logic(triggered_bins_channels,
                                                                            number_coincidences, coinc_window, dt)

        # set maximum signal amplitude
        max_signal = 0

        trigger = EnvelopeTrigger(trigger_name, passband, order, threshold, number_coincidences, coinc_window, triggered_channels)
        trigger.set_triggered_channels(channels_that_passed_trigger)

        if has_triggered:
            trigger.set_triggered(True)
            trigger.set_trigger_time(triggered_times.min() + channel_trace_start_time)  # trigger_time = time from the beginning of the trace
            logger.debug("station has triggered")

        else:
            trigger.set_triggered(False)
            trigger.set_trigger_time(None)
            logger.debug("station has NOT triggered")

        station.set_trigger(trigger)
        self.__t += time.time() - t

    def end(self):
        from datetime import timedelta
        logger.setLevel(logging.INFO)
        dt = timedelta(seconds=self.__t)
        logger.info("total time used by this module is {}".format(dt))
        return dt



import logging
import numpy as np
import argparse
import matplotlib.pyplot as plt
import datetime

from NuRadioReco.utilities import units
import NuRadioReco.detector.detector as detector
import NuRadioReco.modules.io.eventReader
import NuRadioReco.modules.io.eventWriter
import NuRadioReco.modules.channelSignalReconstructor

from NuRadioReco.framework.parameters import channelParameters as chp


# Logging level
from NuRadioReco.modules.base import module
logger = module.setup_logger(level=logging.WARNING)

# Parse eventfile as argument
parser = argparse.ArgumentParser(description='NuRadioSim file')
parser.add_argument('inputfilename', type=str,
                    help='path to NuRadioMC simulation result')
parser.add_argument('detectordescription', type=str,
                    help='path to detectordescription')
args = parser.parse_args()


# read in detector positions (this is a dummy detector)
det = detector.Detector(json_filename=args.detectordescription)
det.update(datetime.datetime(2018, 10, 1))

# initialize modules
eventWriter = NuRadioReco.modules.io.eventWriter.eventWriter()
eventReader = NuRadioReco.modules.io.eventReader.eventReader()
channelSignalReconstructor = NuRadioReco.modules.channelSignalReconstructor.channelSignalReconstructor()

# Name outputfile
output_filename = "Simple_reconstruction_results.nur"
eventReader.begin(args.inputfilename)
eventWriter.begin(output_filename, max_file_size=1000)

i_events_saved = 0
for iE, event in enumerate(eventReader.run()):
    logger.info("Event ID {}".format(event.get_id()))
    for st, station in enumerate(event.get_stations()):
        channelSignalReconstructor.run(event, station, det)
        for channel in station.iter_channels():
            signal_to_noise = channel.get_parameter(chp.SNR)['peak_2_peak_amplitude']
            print('Event{}, Station {}, Channel {}, SNR = {:.2f}'.format(event.get_id(), station.get_id(), channel.get_id(), signal_to_noise))
    eventWriter.run(event)
    i_events_saved += 1
    if((i_events_saved % 100) == 0):
        print("saving event {}".format(i_events_saved))




import dash
import json
import plotly
from NuRadioReco.utilities import units
from NuRadioReco.eventbrowser.default_layout import default_layout
import numpy as np
from NuRadioReco.framework.parameters import stationParameters as stnp
from NuRadioReco.framework.parameters import electricFieldParameters as efp
import dash_html_components as html
import dash_core_components as dcc
from dash.dependencies import Input, Output, State
from app import app
import dataprovider
provider = dataprovider.DataProvider()

layout = [
    html.Div([
        html.Div([
            dcc.Graph(id='cr-polarization-zenith')
        ], style={'flex': '1'}),
    ], style={'display': 'flex'})
]

@app.callback(Output('cr-polarization-zenith', 'figure'),
              [Input('filename', 'value'),
               Input('btn-open-file', 'value'),
               Input('event-ids', 'children'),
               Input('station-id-dropdown', 'value')],
              [State('user_id', 'children')])
def plot_cr_polarization_zenith(filename, btn, jcurrent_selection, station_id, juser_id):
    if filename is None or station_id is None:
        return {}
    user_id = json.loads(juser_id)
    ariio = provider.get_arianna_io(user_id, filename)
    traces = []
    keys = ariio.get_header()[station_id].keys()
    pol = []
    pol_exp = []
    zeniths = []
    for i_event in range(ariio.get_n_events()):
        event = ariio.get_event_i(i_event)
        for station in event.get_stations():
            for electric_field in station.get_electric_fields():
                if electric_field.has_parameter(efp.polarization_angle) and electric_field.has_parameter(efp.polarization_angle_expectation) and electric_field.has_parameter(efp.zenith):
                    pol.append(electric_field.get_parameter(efp.polarization_angle))
                    pol_exp.append(electric_field.get_parameter(efp.polarization_angle_expectation))
                    zeniths.append(electric_field.get_parameter(efp.zenith))
    pol = np.array(pol)
    pol = np.abs(pol)
    pol[pol > 0.5 * np.pi] = np.pi - pol[pol > 0.5 * np.pi]
    pol_exp = np.array(pol_exp)
    pol_exp = np.abs(pol_exp)
    pol_exp[pol_exp > 0.5 * np.pi] = np.pi - pol_exp[pol_exp > 0.5 * np.pi]
    zeniths = np.array(zeniths)
    traces.append(plotly.graph_objs.Scatter(
        x=zeniths / units.deg,
        y=np.abs(pol - pol_exp) / units.deg,
        text=[str(x) for x in ariio.get_event_ids()],
        mode='markers',
        customdata=[x for x in range(ariio.get_n_events())],
        opacity=1
    ))

    current_selection = json.loads(jcurrent_selection)
    if current_selection != []:
        for trace in traces:
            trace['selectedpoints'] = current_selection

    return {
        'data': traces,
        'layout': plotly.graph_objs.Layout(
            xaxis={'type': 'linear', 'title': 'zenith angle [deg]'},
            yaxis={'title': 'polarization angle error [deg]', 'range': [0, 90]},
            hovermode='closest'
        )
    }



from __future__ import absolute_import, division, print_function, unicode_literals
import NuRadioReco.framework.base_trace
import NuRadioReco.framework.trigger
import NuRadioReco.framework.electric_field
import NuRadioReco.framework.parameters as parameters
import datetime
import astropy.time
import NuRadioReco.framework.parameter_serialization
try:
    import cPickle as pickle
except ImportError:
    import pickle
import logging
import collections
logger = logging.getLogger('BaseStation')


class BaseStation():

    def __init__(self, station_id):
        self._parameters = {}
        self._ARIANNA_parameters = {}
        self._parameter_covariances = {}
        self._station_id = station_id
        self._station_time = None
        self._triggers = collections.OrderedDict()
        self._triggered = False
        self._electric_fields = []
        self._particle_type = ''

    def __setitem__(self, key, value):
        self.set_parameter(key, value)

    def __getitem__(self, key):
        return self.get_parameter(key)

    def get_parameter(self, key):
        if not isinstance(key, parameters.stationParameters):
            logger.error("parameter key needs to be of type NuRadioReco.framework.parameters.stationParameters")
            raise ValueError("parameter key needs to be of type NuRadioReco.framework.parameters.stationParameters")
        return self._parameters[key]

    def get_parameters(self):
        return self._parameters

    def has_parameter(self, key):
        if not isinstance(key, parameters.stationParameters):
            logger.error("parameter key needs to be of type NuRadioReco.framework.parameters.stationParameters")
            raise ValueError("parameter key needs to be of type NuRadioReco.framework.parameters.stationParameters")
        return key in self._parameters.keys()

    def set_parameter(self, key, value):
        if not isinstance(key, parameters.stationParameters):
            logger.error("parameter key needs to be of type NuRadioReco.framework.parameters.stationParameters")
            raise ValueError("parameter key needs to be of type NuRadioReco.framework.parameters.stationParameters")
        self._parameters[key] = value

    def set_parameter_error(self, key, value):
        if not isinstance(key, parameters.stationParameters):
            logger.error("parameter key needs to be of type NuRadioReco.framework.parameters.stationParameters")
            raise ValueError("parameter key needs to be of type NuRadioReco.framework.parameters.stationParameters")
        self._parameter_covariances[(key, key)] = value ** 2

    def get_parameter_error(self, key):
        if not isinstance(key, parameters.stationParameters):
            logger.error("parameter key needs to be of type NuRadioReco.framework.parameters.stationParameters")
            raise ValueError("parameter key needs to be of type NuRadioReco.framework.parameters.stationParameters")
        return self._parameter_covariances[(key, key)] ** 0.5

    def remove_parameter(self, key):
        if not isinstance(key, parameters.stationParameters):
            logger.error("parameter key needs to be of type NuRadioReco.framework.parameters.stationParameters")
            raise ValueError("parameter key needs to be of type NuRadioReco.framework.parameters.stationParameters")
        self._parameters.pop(key, None)

    def set_station_time(self, time):
        if isinstance(time, datetime.datetime):
            self._station_time = astropy.time.Time(time)
        else:
            self._station_time = time

    def get_station_time(self):
        return self._station_time

#     def get_trace(self):
#         return self._time_trace
#
#     def set_trace(self, trace, sampling_rate):
#         self._time_trace = trace
#         self._sampling_rate = sampling_rate
#
#     def get_sampling_rate(self):
#         return self._sampling_rate
#
#     def get_times(self):
#         return np.arange(0, len(self._time_trace) / self._sampling_rate, 1. / self._sampling_rate)

    def get_id(self):
        return self._station_id

    def remove_triggers(self):
        self._triggers = collections.OrderedDict()

    def get_trigger(self, name):
        if(name not in self._triggers):
            raise ValueError("trigger with name {} not present".format(name))
        return self._triggers[name]

    def has_trigger(self, trigger_name):
        """
        checks if station has a trigger with a certain name

        Paramters
        ---------
        trigger_name: string
            the name of the trigger

        Returns bool
        """
        return trigger_name in self._triggers

    def get_triggers(self):
        """
        returns a dictionary of the triggers. key is the trigger name, value is a trigger object
        """
        return self._triggers

    def set_trigger(self, trigger):
        if(trigger.get_name() in self._triggers):
            logger.warning(
                "station has already a trigger with name {}. The previous trigger will be overridden!".format(trigger.get_name()))
        self._triggers[trigger.get_name()] = trigger
        self._triggered = trigger.has_triggered() or self._triggered

    def has_triggered(self, trigger_name=None):
        """
        convenience function.

        Parameters
        ----------
        trigger_name: string or None (default None)
            * if None: The function returns False if not trigger was set. If one or multiple triggers were set,
                       it returns True if any of those triggers triggered
            * if trigger name is set: return if the trigger with name 'trigger_name' has a trigger
        """
        if(trigger_name is None):
            return self._triggered
        else:
            return self.get_trigger(trigger_name).has_triggered()

    def set_triggered(self, triggered=True):
        """
        convenience function to set a simple trigger. The recommended interface is to set triggers through the
        set_trigger() interface.
        """
        if(len(self._triggers) > 1):
            raise ValueError("more then one trigger were set. Request is ambiguous")
        trigger = NuRadioReco.framework.trigger.Trigger('default')
        trigger.set_triggered(triggered)
        self.set_trigger(trigger)

    def set_electric_fields(self, electric_fields):
        self._electric_fields = electric_fields

    def get_electric_fields(self):
        return self._electric_fields

    def add_electric_field(self, electric_field):
        self._electric_fields.append(electric_field)

    def get_electric_fields_for_channels(self, channel_ids=None, ray_path_type=None):
        for e_field in self._electric_fields:
            channel_ids2 = channel_ids
            if (channel_ids is None):
                channel_ids2 = e_field.get_channel_ids()
            if e_field.has_channel_ids(channel_ids2):
                if ray_path_type is None:
                    yield e_field
                elif ray_path_type == e_field.get_parameter(parameters.electricFieldParameters.ray_path_type):
                    yield e_field

    def is_neutrino(self):
        if self._particle_type == '':
            logger.error("Stations particle type has not been set")
            raise ValueError("Stations particle type has not been set")

        return self._particle_type == 'nu'

    def is_cosmic_ray(self):
        if self._particle_type == '':
            logger.error("Stations particle type has not been set")
            raise ValueError("Stations particle type has not been set")

        return self._particle_type == 'cr'

    def set_is_neutrino(self):
        """
        set station type to neutrino
        """
        self._particle_type = 'nu'

    def set_is_cosmic_ray(self):
        """
        set station type to cosmic rays (relevant e.g. for refraction into the snow)
        """
        self._particle_type = 'cr'


    # provide interface to ARIANNA specific parameters
    def get_ARIANNA_parameter(self, key):
        if not isinstance(key, parameters.ARIANNAParameters):
            logger.error("parameter key needs to be of type NuRadioReco.framework.parameters.ARIANNAParameters")
            raise ValueError("parameter key needs to be of type NuRadioReco.framework.parameters.ARIANNAParameters")
        return self._ARIANNA_parameters[key]

    def get_ARIANNA_parameters(self):
        return self._ARIANNA_parameters

    def has_ARIANNA_parameter(self, key):
        if not isinstance(key, parameters.ARIANNAParameters):
            logger.error("parameter key needs to be of type NuRadioReco.framework.parameters.ARIANNAParameters")
            raise ValueError("parameter key needs to be of type NuRadioReco.framework.parameters.ARIANNAParameters")
        return key in self._ARIANNA_parameters.keys()

    def set_ARIANNA_parameter(self, key, value):
        if not isinstance(key, parameters.ARIANNAParameters):
            logger.error("parameter key needs to be of type NuRadioReco.framework.parameters.ARIANNAParameters")
            raise ValueError("parameter key needs to be of type NuRadioReco.framework.parameters.ARIANNAParameters")
        self._ARIANNA_parameters[key] = value



    def serialize(self, mode):
        trigger_pkls = []
        for trigger in self._triggers.values():
            trigger_pkls.append(trigger.serialize())
        efield_pkls = []
        if(mode == 'full'):
            for efield in self.get_electric_fields():
                efield_pkls.append(efield.serialize(self))
        data = {'_parameters': NuRadioReco.framework.parameter_serialization.serialize(self._parameters),
                '_parameter_covariances': self._parameter_covariances,
                '_ARIANNA_parameters': self._ARIANNA_parameters,
                '_station_id': self._station_id,
                '_station_time': self._station_time,
                '_particle_type': self._particle_type,
                'triggers': trigger_pkls,
                '_triggered': self._triggered,
                'electric_fields': efield_pkls}
        return pickle.dumps(data, protocol=4)

    def deserialize(self, data_pkl):
        data = pickle.loads(data_pkl)

        if ('triggers' in data):
            self._triggers = NuRadioReco.framework.trigger.deserialize(data['triggers'])

        if ('triggers' in data):
            self._triggered = data['_triggered']

        for electric_field in data['electric_fields']:
            efield = NuRadioReco.framework.electric_field.ElectricField([])
            efield.deserialize(electric_field)
            self.add_electric_field(efield)

        self._parameters = NuRadioReco.framework.parameter_serialization.deserialize(data['_parameters'],
                                    parameters.stationParameters)

        self._parameter_covariances = data['_parameter_covariances']
        if('_ARIANNA_parameters') in data:
            self._ARIANNA_parameters = data['_ARIANNA_parameters']

        self._station_id = data['_station_id']
        self.set_station_time(data['_station_time'])
        self._particle_type = data['_particle_type']



import dash
import json
import plotly
from NuRadioReco.utilities import units
from NuRadioReco.eventbrowser.default_layout import default_layout
import dash_html_components as html
import dash_core_components as dcc
from dash.dependencies import Input, Output, State
from app import app
import dataprovider
provider = dataprovider.DataProvider()

layout = [
    dcc.Graph(id='efield-trace')
]

@app.callback(
    dash.dependencies.Output('efield-trace', 'figure'),
    [dash.dependencies.Input('trigger-trace', 'children'),
     dash.dependencies.Input('event-counter-slider', 'value'),
     dash.dependencies.Input('filename', 'value'),
     dash.dependencies.Input('station-id-dropdown', 'value')],
     [State('user_id', 'children')])
def update_time_efieldtrace(trigger, evt_counter, filename, station_id, juser_id):
    if filename is None or station_id is None:
        return {}
    user_id = json.loads(juser_id)
    colors = plotly.colors.DEFAULT_PLOTLY_COLORS
    ariio = provider.get_arianna_io(user_id, filename)
    evt = ariio.get_event_i(evt_counter)
    station = evt.get_station(station_id)
    fig = plotly.subplots.make_subplots(rows=1, cols=1)
    for electric_field in station.get_electric_fields():
        if electric_field.get_trace() is None:
            trace = np.array([[],[],[]])
        else:
            trace = electric_field.get_trace()
        fig.append_trace(plotly.graph_objs.Scatter(
                x=electric_field.get_times() / units.ns,
                y=trace[0] / units.mV * units.m,
                # text=df_by_continent['country'],
                # mode='markers',
                opacity=0.7,
                marker={
                    'color': colors[0],
                    'line': {'color': colors[0]}
                },
                name='eR'
            ), 1, 1)
        fig.append_trace(plotly.graph_objs.Scatter(
                x=electric_field.get_times() / units.ns,
                y=trace[1] / units.mV * units.m,
                # text=df_by_continent['country'],
                # mode='markers',
                opacity=0.7,
                marker={
                    'color': colors[1],
                    'line': {'color': colors[1]}
                },
                name='eTheta'
            ), 1, 1)
        fig.append_trace(plotly.graph_objs.Scatter(
                x=electric_field.get_times() / units.ns,
                y=trace[2] / units.mV * units.m,
                # text=df_by_continent['country'],
                # mode='markers',
                opacity=0.7,
                marker={
                    'color': colors[2],
                    'line': {'color': colors[2]}
                },
                name='ePhi'
            ), 1, 1)
    fig['layout']['xaxis1'].update(title='time [ns]')
    fig['layout']['yaxis1'].update(title='efield [mV/m]')
    fig['layout'].update(default_layout)
    return fig



from __future__ import absolute_import, division, print_function
import NuRadioReco.framework.parameters as parameters
import NuRadioReco.framework.parameter_serialization
import pickle
import logging
logger = logging.getLogger('Shower')


class BaseShower:

    def __init__(self):
        self._parameters = {}

    def __setitem__(self, key, value):
        self.set_parameter(key, value)

    def __getitem__(self, key):
        return self.get_parameter(key)

    def get_parameter(self, key):
        if not isinstance(key, parameters.showerParameters):
            logger.error("parameter key needs to be of type NuRadioReco.framework.parameters.showerParameters")
            raise ValueError("parameter key needs to be of type NuRadioReco.framework.parameters.showerParameters")
        return self._parameters[key]

    def set_parameter(self, key, value):
        if not isinstance(key, parameters.showerParameters):
            logger.error("parameter key needs to be of type NuRadioReco.framework.parameters.showerParameters")
            raise ValueError("parameter key needs to be of type NuRadioReco.framework.parameters.showerParameters")
        self._parameters[key] = value

    def has_parameter(self, key):
        if not isinstance(key, parameters.showerParameters):
            logger.error("parameter key needs to be of type NuRadioReco.framework.parameters.showerParameters")
            raise ValueError("parameter key needs to be of type NuRadioReco.framework.parameters.showerParameters")
        return key in self._parameters

    def serialize(self):
        data = {'_parameters': NuRadioReco.framework.parameter_serialization.serialize(self._parameters)}
        return pickle.dumps(data, protocol=4)

    def deserialize(self, data_pkl):
        data = pickle.loads(data_pkl)
        self._parameters = NuRadioReco.framework.parameter_serialization.deserialize(data['_parameters'],
                                    parameters.showerParameters)



#!/usr/bin/env python
import argparse
import numpy as np
import NuRadioReco.modules.efieldToVoltageConverter
import NuRadioReco.modules.trigger.simpleThreshold
import NuRadioReco.modules.channelResampler
from NuRadioReco.utilities import units
from NuRadioMC.simulation import simulation


# initialize detector sim modules
efieldToVoltageConverter = NuRadioReco.modules.efieldToVoltageConverter.efieldToVoltageConverter()
efieldToVoltageConverter.begin()
simpleThreshold = NuRadioReco.modules.trigger.simpleThreshold.triggerSimulator()
channelResampler = NuRadioReco.modules.channelResampler.channelResampler()

np.random.seed(1)
class mySimulation(simulation.simulation):


    def _detector_simulation(self):
        # start detector simulation

        efieldToVoltageConverter.run(self._evt, self._station, self._det)  # convolve efield with antenna pattern
        # downsample trace to internal simulation sampling rate (the efieldToVoltageConverter upsamples the trace to
        # 20 GHz by default to achive a good time resolution when the two signals from the two signal paths are added)
        channelResampler.run(self._evt, self._station, self._det, sampling_rate=1. / self._dt)

        simpleThreshold.run(self._evt, self._station, self._det,
                             threshold=3. * self._Vrms,
                             triggered_channels=None,  # run trigger on all channels
                             number_concidences=1,
                             trigger_name='simple_threshold')  # the name of the trigger
        channelResampler.run(self._evt, self._station, self._det, sampling_rate=self._sampling_rate_detector)


sim = mySimulation(inputfilename='NuRadioReco/test/trigger_tests/trigger_test_eventlist.hdf5',
        outputfilename='input.hdf5',
        detectorfile='NuRadioReco/test/trigger_tests/trigger_test_detector.json',
        outputfilenameNuRadioReco='NuRadioReco/test/trigger_tests/trigger_test_input.nur',
        config_file='NuRadioReco/test/trigger_tests/config.yaml',
        file_overwrite=True)
sim.run()



import numpy as np
import scipy.signal

from NuRadioReco.utilities import units, fft
from NuRadioReco.utilities import trace_utilities


def amp_from_energy(energy):
    """
    energy is defined as the integral of squared voltage normalized to a time window of 128 ns

    Parameters
    ----------
    energy:


    """
    return 0.5 * np.log10(energy) + 0.12876705


def get_analytic_pulse_freq(amp_p0, amp_p1, phase_p0, n_samples_time, sampling_rate,
                            phase_p1=0, bandpass=None, quadratic_term=0, quadratic_term_offset = 0):
    """
    Analytic pulse as described in PhD thesis Glaser and NuRadioReco paper in the frequency domain

    Parameters
    ----------
    amp_p0: float
        amplitude parameter of analytic pulse
    amp_p1:
        slope parameter of analytic pulse
    phase_p0:
        phase parameter of analytic pulse
    n_samples_time:
        numer of samples in time-domain
    sampling_rate:
        sampling rate of trace
    phase_p1:
        default 0

    bandpass:
        default None

    quadratic_term:
        default 0

    quadratic_term_offset:
        default 0

    """
    amp_p0 /= trace_utilities.conversion_factor_integrated_signal  # input variable is energy in eV/m^2
    dt = 1. / sampling_rate
    frequencies = np.fft.rfftfreq(n_samples_time, dt)
    df = frequencies[1] - frequencies[0]
    A = np.sign(amp_p0) * (np.abs(amp_p0)) ** 0.5
    amps = A * 10 ** (frequencies * amp_p1 + quadratic_term * (frequencies - quadratic_term_offset)**2)
    norm = 1.
    if(bandpass is None):
        norm = -1. / (2 * amp_p1 * np.log(10))
    else:
        if(amp_p1 == 0):
            norm = bandpass[1] - bandpass[0]
        else:
            norm = (100 ** (amp_p1 * bandpass[1]) - 100 ** (amp_p1 * bandpass[0])) / (2 * amp_p1 * np.log(10))

    phases = phase_p0 + frequencies * phase_p1
    xx = amps * np.exp(phases * 1j) / norm ** 0.5 / dt ** 0.5 * df ** 0.5

    if(bandpass is not None):
        b, a = scipy.signal.butter(10, bandpass, 'bandpass', analog=True)
        w, h = scipy.signal.freqs(b, a, frequencies)
        xx *= h
    return xx


def get_analytic_pulse(amp_p0, amp_p1, phase_p0, n_samples_time,
                       sampling_rate,
                       phase_p1=0, bandpass=None,
                       quadratic_term=0, quadratic_term_offset=0):
    """
    Analytic pulse as described in PhD thesis Glaser and NuRadioReco paper in the time domain

    Parameters
    ----------
    amp_p0: float
        amplitude parameter of analytic pulse
    amp_p1:
        slope parameter of analytic pulse
    phase_p0:
        phase parameter of analytic pulse
    n_samples_time:
        numer of samples in time-domain
    sampling_rate:
        sampling rate of trace
    phase_p1:
        default 0

    bandpass:
        default None

    quadratic_term:
        default 0

    quadratic_term_offset:
        default 0

    """
    xx = get_analytic_pulse_freq(amp_p0, amp_p1, phase_p0, n_samples_time,
                                 sampling_rate, phase_p1=phase_p1,
                                 bandpass=bandpass,
                                 quadratic_term=quadratic_term,
                                 quadratic_term_offset=quadratic_term_offset)
    return fft.freq2time(xx, sampling_rate)



import NuRadioReco.framework.event
from NuRadioReco.modules.base.module import register_run
import NuRadioReco.framework.station
import NuRadioReco.framework.channel
import ROOT
import numpy as np
from NuRadioReco.utilities import units
import sys
import os
import logging
import time
from scipy import interpolate


sys.path.append(os.path.expandvars('${ARA_UTIL_INSTALL_DIR}/lib'))

from ctypes import *
libc = CDLL("libAraEvent.so")
libc = CDLL("libAraConfig.so")
libc = CDLL("libAraCorrelator.so")
libc = CDLL("libAraDisplay.so")
libc = CDLL("libAraKvp.so")
libc = CDLL("libRootFftwWrapper.so")


class readARAData:

    """
    This is the AraReader. Reads ARA data in the ARARoot format.


    """
    def __init__(self):
        self.logger = logging.getLogger("NuRadioReco.readARAData")


    def begin(self, input_file):

        """
        Begin function of the ARA reader

        Parameters
        ----------
        input_file: string
        path to file to read

        """

        self.__id_current_event = -1
        self.__t = time.time()

        self.f = ROOT.TFile.Open(input_file)
        self.raw_ptr = ROOT.RawAtriStationEvent()
        self.data_tree = self.f.Get("eventTree")
        self.data_tree.SetBranchAddress("event", self.raw_ptr)
        self.n_events = self.data_tree.GetEntries()

        return self.n_events

    @register_run()
    def run(self, n_channels=16, sampling=0.625 * units.ns):
        """
        Run function of the ARA reader

        Parameters
        ----------
        n_channels: int
            number of ARA channels to loop over, default 16

        sampling: float (units time)
            default sampling of ARA detector

        """
        while True:
            self.__id_current_event += 1
            if(self.__id_current_event >= self.n_events):
                # all events processed
                break
            if(self.__id_current_event % 1000 == 0):
                progress = 1. * self.__id_current_event / self.n_events
                eta = 0
                if(self.__id_current_event > 0):
                    eta = (time.time() - self.__t) / self.__id_current_event * (self.n_events - self.__id_current_event) / 60.
                self.logger.warning("reading in event {}/{} ({:.0f}%) ETA: {:.1f} minutes".format(self.__id_current_event, self.n_events, 100 * progress, eta))

            self.data_tree.GetEntry(self.__id_current_event)
            run_number = self.data_tree.run
            evt_number = self.raw_ptr.eventNumber
            station_id = bytearray(self.raw_ptr.stationId)[0]
            self.logger.info("Reading Run: {0}, Event {1}, Station {2}".format(run_number, evt_number, station_id))

            evt = NuRadioReco.framework.event.Event(run_number, evt_number)
            station = NuRadioReco.framework.station.Station(station_id)
            real_ptr = ROOT.UsefulAtriStationEvent(self.raw_ptr, ROOT.AraCalType.kLatestCalib)

            # Loop over all channels in data
            for iCh in range(n_channels):

                channel = NuRadioReco.framework.channel.Channel(iCh)

                # Get data from array via graph method
                graph_waveform = real_ptr.getGraphFromRFChan(iCh)
                times = np.array(graph_waveform.GetX()) * units.ns
                voltage = np.array(graph_waveform.GetY()) * units.mV

                # interpolate to get equal sampling between data points
                f_interpolate = interpolate.interp1d(times, voltage)
                times_new = np.arange(times[0], times[-1], sampling)
                voltage_new = f_interpolate(times_new)

                if voltage_new.shape[0] % 2 != 0:
                    voltage_new = voltage_new[:-1]
                sampling_rate = times_new[1] - times_new[0]

                channel.set_trace(voltage_new, sampling_rate)
                station.add_channel(channel)
                evt.set_station(station)
            yield evt

    def end(self):
        pass



from NuRadioReco.utilities import units
import numpy as np
import logging
import scipy.signal
from NuRadioReco.modules.channelGenericNoiseAdder import channelGenericNoiseAdder
from NuRadioReco.utilities.fft import time2freq, freq2time
from scipy.signal import butter, freqs
import NuRadioReco.framework.channel

logger = logging.getLogger('diodeSimulator')

class diodeSimulator:

    def __init__(self, output_passband=(None,None)):
        """
        Calculate a signal as processed by the tunnel diode.
        The given signal is convolved with the tunnel diodde response as in
        AraSim.

        Parameters
        ----------
        output_passband: (float, float) tuple
            Frequencies for a 6th-order Butterworth filter to be applied after
            the diode filtering. If a lowpass filter is needed, please pass a
            (None, float) tuple as parameter.
        """

        logger.info("This module does not contain cutting the trace to ARA specific parameters.")
        logger.info("The user should take care of the appropriate noise rate and trace window.")

        self._output_passband = output_passband

    # Tunnel diode response functions pulled from arasim
    # RL (Robert Lahmann) Sept 3, 2018: this is not documented in the arasim code, but it seems most
    # logical to assume that the units of the two middle parameters are seconds and that the
    # other parameters are unitless
    _td_args = {
        'down1': (-0.8, 15e-9 * units.s, 2.3e-9 * units.s, 0),
        'down2': (-0.2, 15e-9 * units.s, 4e-9 * units.s, 0),
        'up': (1, 18e-9 * units.s, 7e-9 * units.s, 1e9)
    }
    # Set td_args['up'][0] based on the other args, like in arasim
    _td_args['up'] = (-np.sqrt(2 * np.pi) *
                      (_td_args['down1'][0] * _td_args['down1'][2] +
                       _td_args['down2'][0] * _td_args['down2'][2]) /
                      (2e18 * _td_args['up'][2] ** 3),) + _td_args['up'][1:]

    # Set "down" and "up" functions as in arasim
    @classmethod
    def _td_fdown1(cls, x):
        return (cls._td_args['down1'][3] + cls._td_args['down1'][0] *
                np.exp(-(x - cls._td_args['down1'][1]) ** 2 /
                       (2 * cls._td_args['down1'][2] ** 2)))

    @classmethod
    def _td_fdown2(cls, x):
        return (cls._td_args['down2'][3] + cls._td_args['down2'][0] *
                np.exp(-(x - cls._td_args['down2'][1]) ** 2 /
                       (2 * cls._td_args['down2'][2] ** 2)))

    @classmethod
    def _td_fup(cls, x):
        return (cls._td_args['up'][0] *
                (cls._td_args['up'][3] * (x - cls._td_args['up'][1])) ** 2 *
                np.exp(-(x - cls._td_args['up'][1]) / cls._td_args['up'][2]))

    def tunnel_diode(self, channel):
        """
        Calculate a signal as processed by the tunnel diode.
        The given signal is convolved with the tunnel diode response as in
        AraSim.

        The diode model used in this module returns a dimensionless power trace,
        where the antenna resistance is only a normalisation for the final
        voltage. That's why the antenna resistance has been fixed to a value
        of 8.5 ohms.

        Parameters
        ----------
        channel: Channel
            Signal to be processed by the tunnel diode.
        output_passband: (float, float) tuple
            Frequencies for a 6th-order Butterworth filter to be applied after
            the diode filtering.

        Returns
        -------
        trace_after_tunnel_diode: array
            Signal output of the tunnel diode for the input `channel`.
            Careful! This trace is dimensionless and comes from a convolution
            of the power with the diode response.
        """
        t_max = 1e-7 * units.s
        antenna_resistance = 8.5 * units.ohm
        n_pts = int(t_max * channel.get_sampling_rate())
        times = np.linspace(0, t_max, n_pts + 1)
        diode_resp = self._td_fdown1(times) + self._td_fdown2(times)
        t_slice = times > self._td_args['up'][1]
        diode_resp[t_slice] += self._td_fup(times[t_slice])
        conv = scipy.signal.convolve(channel.get_trace() ** 2 / antenna_resistance,
                                     diode_resp, mode='full')
        # conv multiplied by dt so that the amplitude stays constant for
        # varying dts (determined emperically, see ARVZAskaryanSignal comments)
        # Setting output
        trace_after_tunnel_diode = conv / channel.get_sampling_rate()
        trace_after_tunnel_diode = trace_after_tunnel_diode[:channel.get_trace().shape[0]]

        # We filter the output if the band is specified
        if self._output_passband != (None, None):

            sampling_rate = channel.get_sampling_rate()
            trace_spectrum = time2freq(trace_after_tunnel_diode, sampling_rate)
            frequencies = np.linspace(0, sampling_rate/2, len(trace_spectrum))
            if self._output_passband[0] is None:
                b, a = butter(6, self._output_passband[1], 'lowpass', analog=True)
            else:
                b, a = butter(6, self._output_passband, 'bandpass', analog=True)
            w, h = freqs(b, a, frequencies)
            trace_after_tunnel_diode = freq2time(h * trace_spectrum, sampling_rate)

        return trace_after_tunnel_diode

    def calculate_noise_parameters(self,
                                   sampling_rate=1*units.GHz,
                                   min_freq=50*units.MHz,
                                   max_freq=1*units.GHz,
                                   amplitude=10*units.microvolt,
                                   type='rayleigh',
                                   n_tries=10000,
                                   n_samples=10000):
        """
        Calculates the mean and the standard deviation for the diode-filtered noise.

        Parameters
        ----------
        sampling_rate: float
            Sampling rate
        min_freq: float
            Minimum frequency of the bandwidth
        max_freq: float
            Maximum frequency of the bandwidth
        amplitude: float
            Voltage amplitude (RMS) for the noise
        type: string
            Noise type
        n_tries: int
            Number of times the calculation is carried out, to get proper
            averages for the mean and the standard deviation.
        n_samples: int
            Number of samples for each individual noise trace

        Returns
        -------
        power_mean: float
            Mean of the diode-filtered noise
        power_std: float
            Standard deviation of the diode-filtered noise
        """

        power_mean_list = []
        power_std_list = []

        for i_try in range(n_tries):

            noise = NuRadioReco.framework.channel.Channel(0)

            long_noise = channelGenericNoiseAdder().bandlimited_noise(min_freq=min_freq,
                                                                      max_freq=max_freq,
                                                                      n_samples=n_samples,
                                                                      sampling_rate=sampling_rate,
                                                                      amplitude=amplitude,
                                                                      type=type)

            noise.set_trace(long_noise, sampling_rate)
            power_noise = self.tunnel_diode(noise)

            power_mean_list.append(np.mean(power_noise))
            power_std_list.append(np.std(power_noise))

        power_mean = np.mean(power_mean_list)
        power_std = np.mean(power_std_list)

        return power_mean, power_std

    def end(self):

        pass



import NuRadioReco.framework.base_shower
import pickle


class RadioShower(NuRadioReco.framework.base_shower.BaseShower):
    def __init__(self, station_ids=None):
        self.__station_ids = station_ids
        super().__init__()

    def get_station_ids(self):
        return self.__station_ids

    def has_station_ids(self, ids):
        for id in ids:
            if id not in self.__station_ids:
                return False
        return True

    def serialize(self):
        base_shower_pickle = NuRadioReco.framework.base_shower.BaseShower.serialize(self)
        data = {
            'station_ids': self.__station_ids,
            'base_shower': base_shower_pickle
        }
        return pickle.dumps(data, protocol=4)

    def deserialize(self, data_pkl):
        data = pickle.loads(data_pkl)
        NuRadioReco.framework.base_shower.BaseShower.deserialize(self, data['base_shower'])
        self.__station_ids = data['station_ids']



import dash
import json
import plotly
import numpy as np
from app import app
import dash_html_components as html
import dash_core_components as dcc
from dash.dependencies import Input, Output, State
from NuRadioReco.utilities import units
import NuRadioReco.eventbrowser.apps.common
import dataprovider
provider = dataprovider.DataProvider()

layout = [html.Div(id='trigger-overview-properties')]

@app.callback(Output('trigger-overview-properties', 'children'),
                [Input('filename', 'value'),
                Input('event-counter-slider', 'value'),
                Input('station-id-dropdown', 'value')],
                [State('user_id', 'children')])
def trigger_overview_properties(filename, evt_counter, station_id, juser_id):
    if filename is None or station_id is None:
        return ''
    user_id = json.loads(juser_id)
    ariio = provider.get_arianna_io(user_id, filename)
    evt = ariio.get_event_i(evt_counter)
    station = evt.get_station(station_id)
    if station is None:
        return []
    reply = []
    for trigger_name in station.get_triggers():
        props = [
        html.Div([
            html.Div('{}'.format(trigger_name), className='custom-table-th')
        ], className='custom-table-row')
        ]
        trigger = station.get_trigger(trigger_name)
        for setting_name in trigger.get_trigger_settings():
            props.append(
                html.Div([
                    html.Div('{}'.format(setting_name), className='custom-table-td'),
                    html.Div('{}'.format(trigger.get_trigger_settings()[setting_name]), className='custom-table-td custom-table-td-last')
                ], className='custom-table-row')
            )
        reply.append(html.Div(props))
    return reply



import numpy as np
import logging
from NuRadioReco.utilities import units
from NuRadioReco.framework.parameters import channelParameters as chp
from NuRadioReco.modules.base.module import register_run


class channelTimeWindow:
    """
    Set trace outside time window to zero.
    """
    def __init__(self):
        self.logger = logging.getLogger("NuRadioReco.channelTimeWindow")
        self.begin()

    def begin(self, debug=False):
        self.__debug = debug
        if(debug):
            logger.setLevel(logging.DEBUG)

    @register_run()
    def run(self, evt, station, det, window=None,
            window_function='rectangular',
            around_pulse=True,
            window_width=50 * units.ns,
            window_rise_time=20 * units.ns
            ):
        """
        Parameters
        -----------
        evt, station, det
            Event, Station, Detector
        window: list or None
            [time_window_start, time_window_end] in which the signal should be kept (in units of time)
        window_function: string
            select window function
            * rectangular
            * hanning
        around_pulse: float or None
            if not None: specifies time interval around 'signal_time',
            if this option is set, 'window' is ignored
        """

        for channel in station.iter_channels():
            times = channel.get_times()
            trace = channel.get_trace()

            if(around_pulse is not None):
                if(not channel.has_parameter(chp.signal_time)):
                    raise AttributeError("channel parameter 'signal_time' is not set. Did you forget to run the channelSignalReconstructor?")
                tmax = channel[chp.signal_time]
                window = [tmax - window_width * 0.5, tmax + window_width * 0.5]
            if(window_function == 'rectangular'):
                trace[np.where(times < window[0])] = 0.
                trace[np.where(times > window[1])] = 0.
            else:
                window_fkt = np.zeros_like(trace)
                i00 = np.argmin(np.abs(times - (window[0] - window_rise_time)))
                i01 = np.argmin(np.abs(times - window[0]))
                i10 = np.argmin(np.abs(times - window[1]))
                i11 = np.argmin(np.abs(times - (window[1] + window_rise_time)))
#                 print(window)
#                 logger.debug("times = {:.2f} {:.2f} {:.2f} {:.2f}".format(times - (window[0] - window_rise_time),
#                                                           times - window[0],
#                                                           times - window[1],
#                                                           times - (window[1] + window_rise_time)))
                logger.debug("indices = {} {} {} {}".format(i00, i01, i10, i11))
                window_fkt[i01:i10] = 1
                if(window_function == 'hanning'):
                    n = i01 - i00
                    window_fkt[i00:i01] = np.hanning(n * 2)[:n]
                    n = i11 - i10
                    window_fkt[i10:i11] = np.hanning(n * 2)[n:]
                else:
                    raise NotImplementedError("window function {} is not implemented".format(window_function))
                if(self.__debug):
                    from matplotlib import pyplot as plt
                    fig, ax = plt.subplots(1, 1)
                    ax.plot(times, window_fkt)
                    plt.show()
                trace *= window_fkt
            channel.set_trace(trace, channel.get_sampling_rate())

    def end(self):
        pass



from __future__ import absolute_import, division, print_function
from six import iteritems
try:
    import cPickle as pickle
except ImportError:
    import pickle


def deserialize(triggers_pkl):
    triggers = {}
    for data_pkl in triggers_pkl:
        trigger_type = pickle.loads(data_pkl)['_trigger_type']
        if(trigger_type == 'default'):
            trigger = Trigger(None)
            trigger.deserialize(data_pkl)
        elif(trigger_type == 'simple_threshold'):
            trigger = SimpleThresholdTrigger(None, None)
            trigger.deserialize(data_pkl)
        elif(trigger_type == 'high_low'):
            trigger = HighLowTrigger(None, None, None, None, None)
            trigger.deserialize(data_pkl)
        elif(trigger_type == 'simple_phased'):
            trigger = SimplePhasedTrigger(None, None)
            trigger.deserialize(data_pkl)
        elif(trigger_type == 'envelope_trigger'):
            trigger = EnvelopeTrigger(None, None, None, None)
            trigger.deserialize(data_pkl)
        else:
            raise ValueError("unknown trigger type")
        triggers[trigger.get_name()] = trigger
    return triggers


class Trigger:
    """
    base class to store different triggers
    """

    def __init__(self, name, channels=None, trigger_type='default'):
        """
        initialize trigger class

        Parameters
        -----------
        name: string
            unique name of the trigger
        channels: array of ints
            the channels that are involved in the trigger
        type: string
            the trigger type
        """
        self._name = name
        self._channels = channels
        self._triggered = False
        self._trigger_time = None
        self._trigger_times = None
        self._trigger_type = trigger_type
        self._triggered_channels = []

    def has_triggered(self):
        """
        returns true if station has trigger, false otherwise
        """
        return self._triggered

    def set_triggered(self, triggered=True):
        """ set the trigger to True or False """
        self._triggered = triggered

    def set_trigger_time(self, time):
        """ set the trigger time

        Parameters
        ----------
        time: float
            the trigger time from the beginning of the trace
        """
        self._trigger_time = time

    def get_trigger_time(self):
        """
        get the trigger time (time with respect to beginning of trace)
        """
        return self._trigger_time

    def set_trigger_times(self, times):
        """ set the trigger times

        Parameters
        ----------
        times: array
            all trigger times
        """
        self._trigger_times = times

    def get_trigger_times(self):
        """
        get the trigger times (time with respect to beginning of trace)
        """
        return self._trigger_times

    def get_name(self):
        """ get trigger name """
        return self._name

    def get_type(self):
        """ get trigger type """
        return self._type

    def get_triggered_channels(self):
        """ get IDs of channels that have triggered """
        return self._triggered_channels

    def set_triggered_channels(self, triggered_channels):
        self._triggered_channels = triggered_channels

    def serialize(self):
        return pickle.dumps(self.__dict__, protocol=4)

    def deserialize(self, data_pkl):
        for key, value in iteritems(pickle.loads(data_pkl)):
            setattr(self, key, value)

    def __str__(self):
        output = ""
        for key, value in iteritems(self.__dict__):
            output += "{}: {}\n".format(key[1:], value)
        return output

    def get_trigger_settings(self):
        output = {}
        for key, value in iteritems(self.__dict__):
            output[key[1:]] = value
        return output


class SimpleThresholdTrigger(Trigger):

    def __init__(self, name, threshold, channels=None, number_of_coincidences=1,
                 channel_coincidence_window=None):
        """
        initialize trigger class

        Parameters
        -----------
        name: string
            unique name of the trigger
        threshold: float
            the threshold
        channels: array of ints or None
            the channels that are involved in the trigger
            default: None, i.e. all channels
        number_of_coincidences: int
            the number of channels that need to fulfill the trigger condition
            default: 1
        channel_coincidence_window: float or None (default)
            the coincidence time between triggers of different channels
        """
        Trigger.__init__(self, name, channels, 'simple_threshold')
        self._threshold = threshold
        self._number_of_coincidences = number_of_coincidences
        self._coinc_window = channel_coincidence_window


class EnvelopePhasedTrigger(Trigger):

    def __init__(self, name, threshold_factor, power_mean, power_std,
                 triggered_channels=None, phasing_angles=None, trigger_delays=None,
                 output_passband=(None, None)):
        """
        initialize trigger class
        Parameters
        -----------
        name: string
            unique name of the trigger
        threshold_factor: float
            the threshold factor
        power_mean: float
            mean of the noise trace after being filtered with the diode
        power_std: float
            standard deviation of the noise trace after being filtered with the
            diode. power_mean and power_std can be calculated with the function
            calculate_noise_parameters from utilities.diodeSimulator
        triggered_channels: array of ints or None
            the channels that are involved in the main phased beam
            default: None, i.e. all channels
        phasing_angles: array of floats or None
            the angles for each beam
        trigger_delays: dictionary
            the delays for the channels that have caused a trigger.
            If there is no trigger, it's an empty dictionary
        output_passband: (float, float) tuple
            Frequencies for a 6th-order Butterworth filter to be applied after
            the diode filtering.
        """
        Trigger.__init__(self, name, triggered_channels, 'envelope_phased')
        self._triggered_channels = triggered_channels
        self._phasing_angles = phasing_angles
        self._threshold_factor = threshold_factor
        self._power_mean = power_mean
        self._power_std = power_std
        self._trigger_delays = trigger_delays
        self._output_passband = output_passband


class SimplePhasedTrigger(Trigger):

    def __init__(self, name, threshold, channels=None, secondary_channels=None,
                 primary_angles=None, secondary_angles=None,
                 trigger_delays=None, sec_trigger_delays=None):
        """
        initialize trigger class
        Parameters
        -----------
        name: string
            unique name of the trigger
        threshold: float
            the threshold
        channels: array of ints or None
            the channels that are involved in the main phased beam
            default: None, i.e. all channels
        secondary_channels: array of ints or None
            the channels involved in the secondary phased beam
        primary_angles: array of floats or None
            the angles for each subbeam of the primary phasing
        secondary_angles: array of floats or None
            the angles for each subbeam of the secondary phasing
        trigger_delays: dictionary
            the delays for the primary channels that have caused a trigger.
            If there is no trigger, it's an empty dictionary
        sec_trigger_delays: dictionary
            the delays for the secondary channels that have caused a trigger.
            If there is no trigger or no secondary channels, it's an empty dictionary
        """
        Trigger.__init__(self, name, channels, 'simple_phased')
        self._primary_channels = channels
        self._primary_angles = primary_angles
        self._secondary_channels = secondary_channels
        self._secondary_angles = secondary_angles
        self._threshold = threshold
        self._trigger_delays = trigger_delays
        self._sec_trigger_delays = sec_trigger_delays


class HighLowTrigger(Trigger):

    def __init__(self, name, threshold_high, threshold_low, high_low_window,
                 channel_coincidence_window, channels=None, number_of_coincidences=1):
        """
        initialize trigger class

        Parameters
        -----------
        name: string
            unique name of the trigger
        threshold_high: float
            the high threshold
        threshold_low: float
            the low threshold
        high_low_window: float
            the coincidence time between a high and low per channel
        channel_coincidence_window: float
            the coincidence time between triggers of different channels
        channels: array of ints or None
            the channels that are involved in the trigger
            default: None, i.e. all channels
        number_of_coincidences: int
            the number of channels that need to fulfill the trigger condition
            default: 1
        """
        Trigger.__init__(self, name, channels, 'high_low')
        self._number_of_coincidences = number_of_coincidences
        self._threshold_high = threshold_high
        self._threshold_low = threshold_low
        self._high_low_window = high_low_window
        self._coinc_window = channel_coincidence_window


class IntegratedPowerTrigger(Trigger):

    def __init__(self, name, threshold, channel_coincidence_window, channels=None, number_of_coincidences=1,
                 power_mean=None, power_std=None, integration_window=None):
        """
        initialize trigger class

        Parameters
        -----------
        name: string
            unique name of the trigger
        threshold: float
            the threshold
        channel_coincidence_window: float
            the coincidence time between triggers of different channels
        channels: array of ints or None
            the channels that are involved in the trigger
            default: None, i.e. all channels
        number_of_coincidences: int
            the number of channels that need to fulfill the trigger condition
            default: 1
        """
        Trigger.__init__(self, name, channels, 'int_power')
        self._number_of_coincidences = number_of_coincidences
        self._threshold = threshold
        self._coinc_window = channel_coincidence_window
        self._power_mean = power_mean
        self._power_std = power_std
        self._integration_window = integration_window


class EnvelopeTrigger(Trigger):

    def __init__(self, name, passband, order, threshold, number_of_coincidences=2,
                 channel_coincidence_window=None, channels=None):
        """
        initialize trigger class

        Parameters
        -----------
        name: string
            unique name of the trigger
        passband: array
            the passband in which the trigger should apply
        order: int
            order of filtertype 'butterabs'
        threshold: float
            the threshold
        channels: array of ints or None
            the channels that are involved in the trigger
            default: None, i.e. all channels
        number_of_coincidences: int
            the number of channels that need to fulfill the trigger condition
            default: 1
        channel_coincidence_window: float or None (default)
            the coincidence time between triggers of different channels
        """
        Trigger.__init__(self, name, channels, 'envelope_trigger')
        self._passband = passband
        self._order = order
        self._threshold = threshold
        self._number_of_coincidences = number_of_coincidences
        self._coinc_window = channel_coincidence_window



from __future__ import absolute_import, division, print_function  # , unicode_literals
import dash_html_components as html
from NuRadioReco.utilities import units
from NuRadioReco.framework.parameters import stationParameters as stnp
from NuRadioReco.framework.parameters import channelParameters as chp
from NuRadioReco.framework.parameters import electricFieldParameters as efp
from NuRadioReco.eventbrowser.default_layout import default_layout
import NuRadioReco.eventbrowser.apps.simulation_plots.sim_electric_field_trace
import NuRadioReco.eventbrowser.apps.simulation_plots.sim_electric_field_spectrum
import NuRadioReco.eventbrowser.apps.simulation_plots.sim_event_overview
import numpy as np
import logging
logger = logging.getLogger('traces')


layout = html.Div([
    html.Div([
        html.Div([
            html.Div('Sim Traces', className='panel-heading'),
                html.Div(NuRadioReco.eventbrowser.apps.simulation_plots.sim_electric_field_trace.layout,
                className='panel-body', style={'min-height': '500px'})
            ], className='panel panel-default mb-2', style={'flex': '1'}),
            html.Div([
                html.Div('Sim Spectrum', className='panel-heading'),
                html.Div(NuRadioReco.eventbrowser.apps.simulation_plots.sim_electric_field_spectrum.layout,
                className='panel-body', style={'min-height': '500px'})
            ], className='panel panel-default mb-2', style={'flex': '1'})
        ], style={'display': 'flex'}),
        html.Div([
            html.Div([
                html.Div('Simulated Event', className='panel-heading'),
                html.Div(NuRadioReco.eventbrowser.apps.simulation_plots.sim_event_overview.layout,
                className='panel-body', style={'display': 'flex'})
            ], className='panel panel-default', style = {'flex': '1'})
        ], style={'display': 'flex'})
    ])



"""
This file generates the input files for the electron neutrino SNR
curves simulation using a phased array. For creating a test file, run:

python T01generate_event_list.py

The file will create several folders containing neutrinos that interact at a
given angle with respect to the horizon. The

WARNING: This file needs NuRadioMC installed. https://github.com/nu-radio/NuRadioMC
"""

from __future__ import absolute_import, division, print_function
from NuRadioReco.utilities import units
from NuRadioMC.EvtGen.generator import generate_eventlist_cylinder
import numpy as np
import os

z12 = -200 * units.m
rho = 500 * units.m

obs_angles = np.linspace(-10.,10.,41) * units.deg
# Elevation of the vertex point seen from the middle point of the array

for obs_angle in obs_angles:
	# define simulation volume
	zmin = z12 + rho*np.tan(obs_angle) - 0.1 * units.m
	zmax = z12 + rho*np.tan(obs_angle)
	rmin = rho
	rmax = rho + 0.1 * units.m
	print('Neutrino height', zmax)

	theta_c = 55 * units.deg
	theta_nu = 90*units.deg - theta_c - obs_angle

	phimin = 0.*units.deg
	phimax = 0.1*units.deg

	if (theta_nu < 0):
		theta_nu = -theta_nu
		phimin = 180*units.deg
		phimax = 180.1*units.deg

	thetas = np.array([theta_nu, theta_nu + 0.01*units.deg])
	thetamins = thetas[0:-1]
	thetamaxs = thetas[1:]
	print(thetamins)
	print(thetamaxs)

	Es = np.array([18,18.1])
	Es = 10**Es
	Emins = Es[0:-1]
	Emaxs = Es[1:]

	flavours = [12] # no difference between neutrinos and antineutrinos for us

	nevt = 2e3
	nevt_perfile = 1e3

	for thetamin, thetamax in zip(thetamins, thetamaxs):

		for flavour in flavours:

			folder_angle = "{:.1f}deg_{}".format(thetamin/units.deg,flavour)
			try:
				os.mkdir(folder_angle)
			except:
				pass

			folderlist = []
			for ifolder, Emin,Emax in zip(range(len(Emins)),Emins,Emaxs):
				folderlist.append("{:.2f}_{}_{}_{:.2e}_{:.2e}".format(thetamin/units.deg,flavour,str(ifolder).zfill(2),Emin,Emax))
			for folder in folderlist:
				try:
					os.mkdir(os.path.join(folder_angle, folder))
				except:
					pass

			for folder, Emin, Emax in zip(folderlist, Emins, Emaxs):

				input_dir = os.path.join(folder_angle, folder, 'input')
				try:
					os.mkdir(input_dir)
				except:
					pass
				outname = folder_angle+'/'+folder+'/input/'+folder+'.hdf5'
				print(Emin/units.PeV, Emax/units.PeV)
				print(outname)
				generate_eventlist_cylinder(outname, nevt, Emin, Emax, rmin, rmax, zmin, zmax, full_rmin=rmin, full_rmax=rmax, full_zmin=zmin, full_zmax=zmax, thetamin=thetamin, thetamax=thetamax, phimin=phimin, phimax=phimax, start_event_id=1, flavor=[flavour], n_events_per_file=nevt_perfile, deposited=True)



import numpy as np
from NuRadioReco.utilities import units
from NuRadioReco.utilities.io_utilities import read_pickle
import logging
from six import iteritems
import os


class Templates(object):
    __instance = None

    def __new__(cls, path):
        if Templates.__instance is None:
            Templates.__instance = object.__new__(cls)
        return Templates.__instance

    def __init__(self, path):
        self.__ref_cr_templates = {}
        self.__cr_templates = {}
        self.__cr_template_set = {}
        self.__cr_template_set_full = {}
        self.__ref_nu_templates = {}
        self.__nu_templates = {}
        self.__nu_template_set = {}
        self.__path = path
        self.logger = logging.getLogger("NuRadioReco.Templates")

    def set_template_directory(self, path):
        self.__path = path

    def __load_cr_template(self, station_id):
        path = os.path.join(self.__path, 'templates_cr_station_{}.pickle'.format(station_id))
        if(os.path.exists(path)):
            self.__cr_templates[station_id] = read_pickle(path)
            zen_ref = np.deg2rad(60)
            az_ref = np.deg2rad(0)
            self.__ref_cr_templates[station_id] = self.__cr_templates[station_id][0][zen_ref][az_ref]
        else:
            self.logger.error("template file {} not found".format(path))
            raise IOError

    def get_cr_ref_templates(self, station_id):
        """
        returns one cosmic ray template per channel for the reference direction
        """
        if station_id not in self.__ref_cr_templates.keys():
            self.__load_cr_template(station_id)
        return self.__ref_cr_templates[station_id]

    def get_cr_ref_template(self, station_id):
        """
        returns one cosmic ray template for the reference direction, the same for all channels
        """
        self.logger.info("Getting template for station ID {}".format(station_id))
        if station_id in [51, 52]:
            tmpl = self.get_cr_ref_templates(station_id)[4]  # FIXME: hardcoded that channel 4 is a cosmic-ray sensitive channel
        elif station_id == 32:
            tmpl = self.get_cr_ref_templates(station_id)[1]
        elif station_id == 61:
            tmpl = self.get_cr_ref_templates(station_id)[5]
        else:
            self.logger.error("Provided value for CR senistive channel of station {} in templates.py".format(station_id))
            tmpl = None

        return tmpl

    def get_set_of_cr_templates_full(self, station_id, n=100):
        """
        gets set of n templates to allow for the calculation of average templates
        """
        if station_id not in self.__ref_cr_templates.keys():
            self.__load_cr_template(station_id)
        if self.__cr_template_set_full == {}:
            self.logger.info("Getting set of templates for station ID {}".format(station_id))
            cr_set = {}
            n_tmpl = 0

            for templates in self.__cr_templates[station_id]:
                for zen, zen_tempaltes in templates.items():
                    for az, template in zen_tempaltes.items():
                        cr_set[n_tmpl] = template
                        n_tmpl += 1
                        if n_tmpl >= n:
                            self.__cr_template_set_full = cr_set
                            return self.__cr_template_set_full
            self.logger.warning(f"{n} templates requested but only {n_tmpl} are available. Returning only {n_tmpl} templates.")
            return self.__cr_template_set_full
        else:
            return self.__cr_template_set_full

    def get_set_of_cr_templates(self, station_id, n=100):
        """
        gets set of n templates to allow for the calculation of average templates

        loops first over different coreas pulses with different frequency content
        and then over azimuth angles of 0, 22.5 and 45 degree
        and then over zenith angles of 60, 50 and 70 degree
        """
        if station_id not in self.__ref_cr_templates.keys():
            self.__load_cr_template(station_id)
        if self.__cr_template_set == {}:
            self.logger.info("Getting set of templates for station ID {}".format(station_id))
            n_tmpl = 0
            zen_refs = np.deg2rad([60, 50, 70])
            az_refs = np.deg2rad([0, 22.5, 45])

            for zen in zen_refs:
                for az in az_refs:
                    for templates in self.__cr_templates[station_id]:
                        self.__cr_template_set[n_tmpl] = templates[zen][az]
                        n_tmpl += 1
                        if n_tmpl >= n:
                            break
        return self.__cr_template_set

    def get_set_of_nu_templates(self, station_id, n=100):
        """
        gets set of n templates to allow for the calculation of average templates

        loops first over different viewing angles
        and then over azimuth angles of 0, 22.5 and 45 degree
        and then over zenith angles of 100, 120 and 140 degree
        """
        if station_id not in self.__ref_nu_templates.keys():
            self.__load_nu_template(station_id)
        if self.__nu_template_set == {}:
            self.logger.info("Getting set of templates for station ID {}".format(station_id))
            n_tmpl = 0
            zen_refs = np.deg2rad([100, 120, 140])
            az_refs = np.deg2rad([0, 22.5, 45])
            dCherenkovs = np.deg2rad([0, -0.5, -1, -1.5, -2, -3, -4, -5])

            for zen in zen_refs:
                for az in az_refs:
                    for dCh in dCherenkovs:
                        self.__nu_template_set[n_tmpl] = self.__nu_templates[station_id][zen][az][dCh]
                        n_tmpl += 1
                        if n_tmpl >= n:
                            break
        return self.__nu_template_set

    def __load_nu_template(self, station_id):
        path = os.path.join(self.__path, 'templates_nu_station_{}.pickle'.format(station_id))
        if(os.path.exists(path)):
            self.__nu_templates[station_id] = read_pickle(path)
            zen_ref = np.deg2rad(140)
            az_ref = np.deg2rad(45)
            self.__ref_nu_templates[station_id] = self.__nu_templates[station_id][zen_ref][az_ref]
        else:
            self.logger.error("template file {} not found".format(path))
            raise IOError

    def get_nu_ref_templates(self, station_id):
        """
        returns one neutrino template per channel for the reference direction and on the cherenkov cone
        """
        if station_id not in self.__ref_nu_templates.keys():
            self.__load_nu_template(station_id)
        return self.__ref_nu_templates[station_id][0.0]

    def get_nu_ref_template(self, station_id):
        """
        returns one neutrino template for the reference direction and on the cherenkov cone, the same for all channels
        """
        return self.get_nu_ref_templates(station_id)[0]  # FIXME: hardcoded that channel 0 is a neutrino sensitive channel



import numpy as np
import os
from radiotools import helper as hp
from NuRadioReco.utilities import units
import logging
import pickle
import scipy.signal as ss
from numpy import pi, diff, unwrap, angle
import matplotlib.pyplot  as plt

logger = logging.getLogger('analog_components')


def save_preprocessed_Amps(response, amp_name):
    output_filename = '{}.pkl'.format(amp_name)
    with open(output_filename, 'wb') as fout:
        logger.info('saving output to {}'.format(output_filename))
        pickle.dump(response, fout, protocol=4)

def preprocess_300Amp(amp, channel, path="HardwareResponses/"):
    """
    preprocess individual 300 series measurements

    Parameters
    ----------
    amp: string
        amp series - box number - and channel. Ex: 300-01
    channel: int
        channel id
    path: string
        directory where amp responses are located. Default is "HardwareResponses/"
    """

    # load null measurement
    amp_gain_db = np.loadtxt(os.path.join(path, 'LOGMAG_NULL.CSV'), skiprows=3, delimiter=',')
    ph = os.path.join(path, 'PHASE_NULL.CSV')
    amp_phase_discrete = np.loadtxt(ph, skiprows=3, delimiter=',')

    # Convert to GHz and add 40dB for attenuation in measurement circuit
    amp_gain_db[:, 0] *= units.Hz
    amp_phase_discrete[:, 0] *= units.Hz
    amp_phase_discrete[:, 1] *= units.deg
    if(not np.allclose(amp_gain_db[:, 0], amp_phase_discrete[:, 0])):
        raise ValueError("frequencies of gain and phase measurement are not equal for NULL measurement")
    amp_gain_db[:, 1] += 40  # 300 series amps
    gain_lin = 10 ** (amp_gain_db[:, 1] / 20.)
    response_null = gain_lin * np.exp(1j * amp_phase_discrete[:, 1])


    # read in individual amp measurement
    path = os.path.join(path, 'Box' + amp)
    ch_name = '_CH{:d}.CSV'.format(channel)
    amp_gain_db = np.loadtxt(os.path.join(path, 'LOGMAG' + ch_name), skiprows=3, delimiter=',')
    ph = os.path.join(path, 'PHASE' + ch_name)
    amp_phase_discrete = np.loadtxt(ph, skiprows=3, delimiter=',')

    # Convert to GHz and add 40dB for attenuation in measurement circuit
    amp_gain_db[:, 0] *= units.Hz
    amp_phase_discrete[:, 0] *= units.Hz
    amp_phase_discrete[:, 1] *= units.deg
    ff = amp_gain_db[:, 0]
    if(not np.allclose(amp_gain_db[:, 0], amp_phase_discrete[:, 0])):
        raise ValueError("frequencies of gain and phase measurement are not equal for {}-{:02d}".format(amp, channel))

    amp_gain_db[:, 1] += 40  # 300 series amps

    gain_lin = 10 ** (amp_gain_db[:, 1] / 20.)
    r = gain_lin * np.exp(1j * amp_phase_discrete[:, 1])

    response = {}
    response['freqs'] = ff
    response['response'] = r / response_null
    amplifier_response = {}
    amp_name = '{}-{:02d}'.format(amp, channel)
    amplifier_response[amp_name] = response

    # checkGroupDelay(get_amp_gain(ff)*get_amp_phase(ff),ff,ch_name,path)
    save_preprocessed_Amps(amplifier_response, amp_name)


for i in range(8):
    preprocess_300Amp('300-03', i, path='HardwareResponses/300SeriesAmpBoxes')



"""
This file runs a phased array trigger simulation. The phased array configuration
in this file is similar to one of the proposed ideas for RNO-G: 3 GS/s, 4 antennas
at a depth of ~100 m, 30 primary phasing directions. In order to run, we need
a detector file and a configuration file, included in this folder. To run
the code, type:

python T02RunPhasedRNO_shortband.py input_neutrino_file.hdf5 proposalcompact_100m_1.5GHz.json
config_RNO.yaml output_NuRadioMC_file.hdf5 output_NuRadioReco_file.nur

The antenna positions can be changed in the detector position. The config file
defines de bandwidth for the noise RMS calculation. The properties of the phased
array can be changed in the current file - phasing angles, triggering channels,
bandpass filter and so on.

This file calculates the effective volume for the (140-700) MHz band

WARNING: this file needs NuRadioMC to be run.
"""

from __future__ import absolute_import, division, print_function
import argparse
# import detector simulation modules
import NuRadioReco.modules.efieldToVoltageConverter
import NuRadioReco.modules.trigger.simpleThreshold
import NuRadioReco.modules.envelope_phasedarray.triggerSimulator
import NuRadioReco.modules.channelResampler
import NuRadioReco.modules.channelBandPassFilter
import NuRadioReco.modules.channelGenericNoiseAdder
from NuRadioReco.utilities import units
from NuRadioMC.simulation import simulation
import NuRadioReco.utilities.diodeSimulator
from NuRadioReco.utilities.traceWindows import get_window_around_maximum
import numpy as np
import logging
from NuRadioReco.modules.base import module
logger = module.setup_logger(level=logging.WARNING)

# initialize detector sim modules
efieldToVoltageConverter = NuRadioReco.modules.efieldToVoltageConverter.efieldToVoltageConverter()
efieldToVoltageConverter.begin(debug=False)
triggerSimulator = NuRadioReco.modules.envelope_phasedarray.triggerSimulator.triggerSimulator()
channelResampler = NuRadioReco.modules.channelResampler.channelResampler()
channelBandPassFilter = NuRadioReco.modules.channelBandPassFilter.channelBandPassFilter()
channelGenericNoiseAdder = NuRadioReco.modules.channelGenericNoiseAdder.channelGenericNoiseAdder()
thresholdSimulator = NuRadioReco.modules.trigger.simpleThreshold.triggerSimulator()

main_low_angle = -50 * units.deg
main_high_angle = 50 * units.deg
phasing_angles = np.arcsin( np.linspace( np.sin(main_low_angle), np.sin(main_high_angle), 30) )

diode_passband = (None, 200*units.MHz)
diodeSimulator = NuRadioReco.utilities.diodeSimulator.diodeSimulator(diode_passband)

class mySimulation(simulation.simulation):

    def _detector_simulation(self):
        # start detector simulation
        efieldToVoltageConverter.run(self._evt, self._station, self._det)  # convolve efield with antenna pattern
        # downsample trace to 3 ns
        new_sampling_rate = 3 * units.GHz
        channelResampler.run(self._evt, self._station, self._det, sampling_rate=new_sampling_rate)

        cut_times = get_window_around_maximum(self._station, diodeSimulator, ratio = 0.01)

        # Bool for checking the noise triggering rate
        check_only_noise = False

        if check_only_noise:

            for channel in self._station.iter_channels():  # loop over all channels (i.e. antennas) of the station
                trace = channel.get_trace() * 0
                channel.set_trace(trace, sampling_rate = new_sampling_rate)

        if self._is_simulate_noise():
            max_freq = 0.5 / self._dt
            norm = self._get_noise_normalization(self._station.get_id())  # assuming the same noise level for all stations
            channelGenericNoiseAdder.run(self._evt, self._station, self._det, amplitude=self._Vrms, min_freq=0 * units.MHz,
                                         max_freq=max_freq, type='rayleigh', bandwidth=norm)
        # bandpass filter trace, the upper bound is higher then the sampling rate which makes it just a highpass filter
        channelBandPassFilter.run(self._evt, self._station, self._det, passband=[132 * units.MHz, 1150 * units.MHz],
                                  filter_type='butter', order=8)
        channelBandPassFilter.run(self._evt, self._station, self._det, passband=[0, 700 * units.MHz],
                                  filter_type='butter', order=10)

        # for running an ARA-like trigger, we need to know the filtered noise
        # parameters. REMEMBER TO CHANGE THE FREQUENCIES FOR CALCULATING THEM
        power_mean, power_std = diodeSimulator.calculate_noise_parameters(new_sampling_rate,
                                                                          132 * units.MHz,
                                                                          700 * units.MHz,
                                                                          amplitude=self._Vrms)

        # first run a simple threshold trigger
        triggerSimulator.run(self._evt, self._station, self._det,
                             threshold_factor=3.9, # see envelope phased trigger module for explanation
                             power_mean=power_mean,
                             power_std=power_std,
                             triggered_channels=None,  # run trigger on all channels
                             trigger_name='envelope_phasing', # the name of the trigger
                             phasing_angles=phasing_angles,
                             ref_index=1.75,
                             output_passband=diode_passband,
                             cut_times=cut_times)

parser = argparse.ArgumentParser(description='Run NuRadioMC simulation')
parser.add_argument('--inputfilename', type=str,
                    help='path to NuRadioMC input event list', default='0.00_12_00_1.00e+16_1.00e+19.hdf5')
parser.add_argument('--detectordescription', type=str,
                    help='path to file containing the detector description', default='4antennas_100m_1.5GHz.json')
parser.add_argument('--config', type=str,
                    help='NuRadioMC yaml config file', default='config_RNO.yaml')
parser.add_argument('--outputfilename', type=str,
                    help='hdf5 output filename', default='output_PA_RNO.hdf5')
parser.add_argument('--outputfilenameNuRadioReco', type=str, nargs='?', default=None,
                    help='outputfilename of NuRadioReco detector sim file')
args = parser.parse_args()

sim = mySimulation(inputfilename=args.inputfilename,
                            outputfilename=args.outputfilename,
                            detectorfile=args.detectordescription,
                            outputfilenameNuRadioReco=args.outputfilenameNuRadioReco,
                            config_file=args.config)
sim.run()



import numpy as np
from scipy import constants
from NuRadioReco.utilities import units
from numpy.lib import scimath as SM
import logging
logger = logging.getLogger('NuRadioReco.geometryUtilities')


def get_time_delay_from_direction(zenith, azimuth, positions, n=None):
    """
    Calculate the time delay between given positions for an arrival direction

    Parameters
    ---------

    zenith: float [rad]
        Zenith angle in convention up = 0
    azimuth: float [rad]
        Azimuth angle in convention East = 0, counter-clock-wise
    positions: array[N x 3]
        Positions on ground

    """
    if(n is None):  # assume propagation through air as default
        n = 1.000293
    shower_axis = np.array([np.sin(zenith) * np.cos(azimuth), np.sin(zenith) * np.sin(azimuth), np.cos(zenith)])

    if positions.ndim == 1:
        times = -(1 / (constants.c / n)) * np.dot(shower_axis, positions) * units.s
    else:
        times = np.zeros(positions.shape[0])
        for i in range(positions.shape[0]):
            times[i] = -(1 / (constants.c / n)) * np.dot(shower_axis, positions[i, :]) * units.s

    return times


def rot_z(angle):
    """Angle helper function"""
    m = np.array([  [np.cos(angle), -1 * np.sin(angle), 0],
                    [np.sin(angle), np.cos(angle), 0],
                    [0, 0, 1]
                    ])
    return m


def rot_x(angle):
    """Angle helper function"""
    m = np.array([  [1, 0, 0],
                    [0, np.cos(angle), -1 * np.sin(angle)],
                    [0, np.sin(angle), np.cos(angle)]
                    ])
    return m


def rot_y(angle):
    """Angle helper function"""
    m = np.array([  [np.cos(angle), 0, np.sin(angle)],
                    [0, 1, 0],
                    [-1 * np.sin(angle), 0, np.cos(angle)]
                    ])
    return m


def get_efield_in_spherical_coords(efield, theta, phi):
    """
    Get 3D electric field from cartesian coordinates in spherical coordinates,
    using the arrival directions theta and phi
    """
    ct = np.cos(theta)
    st = np.sin(theta)
    cp = np.cos(phi)
    sp = np.sin(phi)
    e1 = np.array([st * cp, st * sp, ct])
    e2 = np.array([ct * cp, ct * sp, -st])
    e3 = np.array([-sp, cp, 0])
#     e1 /= linalg.norm(e1)
#     e2 /= linalg.norm(e2)
#     e3 /= linalg.norm(e3)

    transformation_matrix = np.matrix([e1, e2, e3])
#     inverse_transformation_matrix = np.linalg.inv(transformation_matrix)

    efield_2 = np.squeeze(np.asarray(np.dot(transformation_matrix, efield)))
    return efield_2


def get_fresnel_angle(zenith_incoming, n_2=1.3, n_1=1.):
    """ Apply Snell's law for given zenith angle, when a signal travels from n1 to n2 """
    t = n_1 / n_2 * np.sin(zenith_incoming)
    if t > 1:
        logger.debug('Fresnel refraction results in unphysical values, refraction from {n1} to {n2} with incoming angle {zenith:.1f}, returning None'.format(n1=n_1, n2=n_2, zenith=np.rad2deg(zenith_incoming)))
        return None
    else:
        if(zenith_incoming > 0.5 * np.pi):
            return np.pi - np.arcsin(t)
        return np.arcsin(t)


def get_fresnel_t_p(zenith_incoming, n_2=1.3, n_1=1.):
    """  returns the coefficient t which is the ratio of the transmitted wave's
    electric field amplitude to that of the incident wave for parallel polarization (p-wave)
    this polarization corresponds to the eTheta polarization

    parallel and perpendicular refers to the signal's polarization with respect
    to the 'plane of incident' which is defindes as: "the plane of incidence
    is the plane which contains the surface normal and the propagation vector
    of the incoming radiation."
    """
    zenith_outgoing = get_fresnel_angle(zenith_incoming, n_2, n_1)
    t = 2 * n_1 * np.cos(zenith_incoming) / (n_1 * np.cos(zenith_outgoing) + n_2 * np.cos(zenith_incoming))
    return t


def get_fresnel_t_s(zenith_incoming, n_2=1.3, n_1=1.):
    """  returns the coefficient t which is the ratio of the transmitted wave's
    electric field amplitude to that of the incident wave for perpendicular polarization (s-wave)
    this polarization corresponds to the ePhi polarization

    parallel and perpendicular refers to the signal's polarization with respect
    to the 'plane of incident' which is defindes as: "the plane of incidence
    is the plane which contains the surface normal and the propagation vector
    of the incoming radiation."
    """
    zenith_outgoing = get_fresnel_angle(zenith_incoming, n_2, n_1)
    t = 2 * n_1 * np.cos(zenith_incoming) / (n_1 * np.cos(zenith_incoming) + n_2 * np.cos(zenith_outgoing))
    return t


def get_fresnel_r_p(zenith_incoming, n_2=1.3, n_1=1.):
    """  returns the coefficient r which is the ratio of the reflected wave's
    electric field amplitude to that of the incident wave for parallel polarization (p-wave)
    this polarization corresponds to the eTheta polarization

    parallel and perpendicular refers to the signal's polarization with respect
    to the 'plane of incident' which is defindes as: "the plane of incidence
    is the plane which contains the surface normal and the propagation vector
    of the incoming radiation."
    """
    n = n_2/n_1
    return (-n**2 * np.cos(zenith_incoming) + SM.sqrt(n**2 - np.sin(zenith_incoming)**2)) / \
              (n**2 * np.cos(zenith_incoming) + SM.sqrt(n**2 - np.sin(zenith_incoming)**2))


def get_fresnel_r_s(zenith_incoming, n_2=1.3, n_1=1.):
    """  returns the coefficient r which is the ratio of the reflected wave's
    electric field amplitude to that of the incident wave for perpendicular polarization (s-wave)
    this polarization corresponds to the ePhi polarization

    parallel and perpendicular refers to the signal's polarization with respect
    to the 'plane of incident' which is defindes as: "the plane of incidence
    is the plane which contains the surface normal and the propagation vector
    of the incoming radiation."
    """
    n = n_2/n_1
    return (np.cos(zenith_incoming) - SM.sqrt(n**2 - np.sin(zenith_incoming)**2)) / \
              (np.cos(zenith_incoming) + SM.sqrt(n**2 - np.sin(zenith_incoming)**2))




import numpy as np
import os, scipy, sys
import copy
import datetime
import glob
import matplotlib.pyplot as plt

from NuRadioReco.utilities import units
from NuRadioReco.detector import detector

import NuRadioReco.modules.io.coreas.readCoREAS
import NuRadioReco.modules.io.coreas.simulationSelector
import NuRadioReco.modules.efieldToVoltageConverter
import NuRadioReco.modules.ARIANNA.hardwareResponseIncorporator
import NuRadioReco.modules.channelGenericNoiseAdder
import NuRadioReco.modules.trigger.simpleThreshold
import NuRadioReco.modules.channelBandPassFilter
import NuRadioReco.modules.electricFieldBandPassFilter
import NuRadioReco.modules.cosmicRayIdentifier
import NuRadioReco.modules.channelStopFilter
import NuRadioReco.modules.channelSignalReconstructor
import NuRadioReco.modules.correlationDirectionFitter
import NuRadioReco.modules.voltageToEfieldConverter
import NuRadioReco.modules.electricFieldSignalReconstructor
import NuRadioReco.modules.voltageToAnalyticEfieldConverter
import NuRadioReco.modules.channelResampler
import NuRadioReco.modules.electricFieldResampler

import NuRadioReco.modules.io.eventWriter

from NuRadioReco.framework.parameters import channelParameters as chp
from NuRadioReco.framework.parameters import stationParameters as stnp

# Logging level
import logging
from NuRadioReco.modules.base import module
logger = module.setup_logger(level=logging.INFO)

plt.switch_backend('agg')


"""
Here, we show an example reconstruction of CoREAS data. A variety of modules
are being used. Please refer to details in the modules themselves.

Input parameters (all with a default provided)
---------------------

Command line input:
    python FullReconstruction.py station_id input_file detector_file

station_id: int
            station id to be used, default 32
input_file: str
            CoREAS simulation file, default example data
detector_file: str
            path to json detector database, default given
"""

try:
    station_id = int(sys.argv[1])  # specify station id
    input_file = sys.argv[2]  # file with coreas simulations
except:
    print("Usage: python SimpleMCReconstruction.py station_id input_file detector")
    station_id = 32
    input_file = "example_data/example_event.h5"
    print("Using default station {}".format(32))

if(station_id == 32):
    triggered_channels = [0, 1, 2, 3]
    used_channels_efield = [0, 1, 2, 3]
    used_channels_fit = [0, 1, 2, 3]
    channel_pairs = ((0, 2), (1, 3))

else:
    print("Default channels not defined for station_id != 32")

try:
    detector_file = sys.argv[3]
    print("Using {0} as detector".format(detector_file))
except:
    print("Using default file for detector")
    detector_file = '../examples/example_data/arianna_detector_db.json'

det = detector.Detector(json_filename=detector_file)  # detector file
det.update(datetime.datetime(2018, 10, 1))

dir_path = os.path.dirname(os.path.realpath(__file__))  # get the directory of this file

# initialize all modules that are needed for processing
# provide input parameters that are to remain constant during processung
readCoREAS = NuRadioReco.modules.io.coreas.readCoREAS.readCoREAS()
readCoREAS.begin([input_file], station_id, n_cores=10, max_distance=None)

simulationSelector = NuRadioReco.modules.io.coreas.simulationSelector.simulationSelector()
simulationSelector.begin()

electricFieldBandPassFilter = NuRadioReco.modules.electricFieldBandPassFilter.electricFieldBandPassFilter()

efieldToVoltageConverter = NuRadioReco.modules.efieldToVoltageConverter.efieldToVoltageConverter()
efieldToVoltageConverter.begin(debug=False)

channelGenericNoiseAdder = NuRadioReco.modules.channelGenericNoiseAdder.channelGenericNoiseAdder()
channelGenericNoiseAdder.begin()

channelBandPassFilter = NuRadioReco.modules.channelBandPassFilter.channelBandPassFilter()
channelBandPassFilter.begin()

channelStopFilter = NuRadioReco.modules.channelStopFilter.channelStopFilter()

voltageToEfieldConverter = NuRadioReco.modules.voltageToEfieldConverter.voltageToEfieldConverter()

electricFieldSignalReconstructor = \
    NuRadioReco.modules.electricFieldSignalReconstructor.electricFieldSignalReconstructor()
electricFieldSignalReconstructor.begin()

voltageToAnalyticEfieldConverter = \
    NuRadioReco.modules.voltageToAnalyticEfieldConverter.voltageToAnalyticEfieldConverter()
voltageToAnalyticEfieldConverter.begin()

cosmicRayIdentifier = NuRadioReco.modules.cosmicRayIdentifier.cosmicRayIdentifier()

channelResampler = NuRadioReco.modules.channelResampler.channelResampler()
channelResampler.begin()

eventWriter = NuRadioReco.modules.io.eventWriter.eventWriter()
output_filename = "MC_example_station_{}.nur".format(station_id)
eventWriter.begin(output_filename)

# Loop over all events in file as initialized in readCoRREAS and perform analysis
for iE, evt in enumerate(readCoREAS.run(detector=det)):

    logger.info("processing event {:d} with id {:d}".format(iE, evt.get_id()))
    station = evt.get_station(station_id)

    if simulationSelector.run(evt, station.get_sim_station(), det):

        cosmicRayIdentifier.run(evt, station, "forced")

        efieldToVoltageConverter.run(evt, station, det)

        channelGenericNoiseAdder.run(evt, station, det, min_freq=25 * units.MHz, type = "rayleigh", amplitude = 1 * units.mV)


        channelStopFilter.run(evt, station, det)
        channelResampler.run(evt, station, det, sampling_rate=0.8 * units.GHz)

        # voltageToAnalyticEfieldConverter expect butter filter
        channelBandPassFilter.run(evt, station, det, passband=[20 * units.MHz, 90 * units.MHz], filter_type='rectangular')
        channelBandPassFilter.run(evt, station, det, passband=[30 * units.MHz, 80 * units.MHz], filter_type='butter', order = 10)

        # traditional
        voltageToEfieldConverter.run(evt, station, det, use_channels=used_channels_efield, use_MC_direction=True)
        electricFieldBandPassFilter.run(evt, station, det, passband=[30 * units.MHz, 80 * units.MHz], filter_type='butter', order=10)
        electricFieldSignalReconstructor.run(evt, station, det)

        # channelSignalReconstructor.run(evt, station, det)
        # new analytic approach
        voltageToAnalyticEfieldConverter.run(evt, station, det, use_channels=used_channels_efield, bandpass=[30*units.MHz, 80*units.MHz], useMCdirection=True)

        eventWriter.run(evt)

nevents = eventWriter.end()
print("Finished processing, {} events".format(nevents))



from NuRadioReco.modules.base.module import register_run
from NuRadioReco.utilities import units
import numpy as np
import logging
import glob
import os
import sys
from NuRadioReco.modules.io import NuRadioRecoio
logger = logging.getLogger('noiseImporter')


class noiseImporter:
    """
    Imports recorded noise from ARIANNA station. The recorded noise needs to match the station geometry and sampling
    as chosen with channelResampler and channelLengthAdjuster

    For different stations, new noise files need to be used.
    Collect forced triggers from any type of station to use for analysis.
    A seizable fraction of data is recommended for accuracy.

    The noise will be random. This module therefore might produce non-reproducible results on a single event basis,
    if run several times.
    """

    def __init__(self):
        self.__channel_mapping = None
        self.__station_id = None

    def begin(self, noise_folder, station_id=None, noise_files=None,
              channel_mapping=None, log_level=logging.WARNING, mean_opt=True):
        """
        Parameters
        ----------
        noise_folder: string
            the folder containing the noise files
        station_id: int
            the station id, specifies from which station the forced triggers are used
            as a noise sample. The data must have the naming convention 'forced_station_??.nur'
            where ?? is replaced with the station id. If station_id is None, the noiseImporter
            will try to find the station with the same iD as the one passed to the run
            function in the noise file
        channel_mapping: dict or None
            option relevant for MC studies of new station designs where we do not
            have forced triggers for. The channel_mapping dictionary maps the channel
            ids of the MC station to the channel ids of the noise data
            Default is None which is 1-to-1 mapping
        log_level: loggging log level
            the log level, default logging.WARNING
        mean_opt: boolean
            option to subtract mean from trace. Set mean_opt=False to remove mean subtraction from trace.
        """
        logger.setLevel(log_level)
        self.__channel_mapping = channel_mapping
        self.__mean_opt = mean_opt
        if(noise_files is not None):
            self.__noise_files = noise_files
        else:
            if(station_id is None):
                logger.error("noise_files and station_id can't be both None")
                sys.exit(-1)
            else:
                self.__noise_files = glob.glob(os.path.join(noise_folder, "forced_station_{}.nur".format(station_id)))
                if(len(self.__noise_files) == 0):
                    logger.error("no noise files found for station {} in folder {}".format(station_id, noise_folder))
                    sys.exit(-1)

        # open files and scan for number of events
        self.__open_files = []
        self.__n_tot = 0
        for file_name in self.__noise_files:
            f = NuRadioRecoio.NuRadioRecoio(file_name, parse_header=False)
            n = f.get_n_events()
            self.__open_files.append({'f': f, 'n_low': self.__n_tot,
                                      'n_high': self.__n_tot + n - 1})
            self.__n_tot += n

    def __get_noise_event(self, i):
        for f in self.__open_files:
            if(i >= f['n_low'] and i <= f['n_high']):
                return f['f'].get_event_i(i - f['n_low'])

    def __get_noise_channel(self, channel_id):
        if(self.__channel_mapping is None):
            return channel_id
        else:
            return self.__channel_mapping[channel_id]

    @register_run()
    def run(self, evt, station, det):
        # loop over stations in simulation
        i_noise = np.random.randint(0, self.__n_tot)
        noise_event = self.__get_noise_event(i_noise)
        if self.__station_id is None:
            station_id = station.get_id()
        else:
            station_id = self.__station_id
        noise_station = noise_event.get_station(station_id)
        if noise_station is None:
            raise KeyError('Station whith ID {} not found in noise file'.format(station_id))
        logger.info("choosing noise event {} ({}, run {}, event {}) randomly".format(i_noise, noise_station.get_station_time(),
                                                                                     noise_event.get_run_number(),
                                                                                     noise_event.get_id()))

        for channel in station.iter_channels():
            channel_id = channel.get_id()

            trace = channel.get_trace()
            noise_channel = noise_station.get_channel(self.__get_noise_channel(channel_id))
            noise_trace = noise_channel.get_trace()
            # check if trace has the same size
            if (len(trace) != len(noise_trace)):
                logger.error("Mismatch: Noise has {0} and simulation {1} samples".format(len(noise_trace), len(trace)))
                sys.exit(-1)
            # check sampling rate
            if (channel.get_sampling_rate() != noise_channel.get_sampling_rate()):
                logger.error("Mismatch in sampling rate: Noise has {0} and simulation {1} GHz".format(
                    noise_channel.get_sampling_rate() / units.GHz, channel.get_sampling_rate() / units.GHz))
                sys.exit(-1)

            trace = trace + noise_trace

            if self.__mean_opt:
                mean = noise_trace.mean()
                std = noise_trace.std()
                if(mean > 0.05 * std):
                    logger.warning(
                        "the noise trace has an offset of {:.2}mV which is more than 5\% of the STD of {:.2f}mV. The module corrects for the offset but it might points to an error in the FPN subtraction.".format(mean, std))
                trace = trace - mean

            channel.set_trace(trace, channel.get_sampling_rate())

    def end(self):
        for f in self.__open_files:
            f['f'].close_file()



import os
import astropy.time
import logging
from tinydb import TinyDB, Query
from tinydb_serialization import SerializationMiddleware
from tinydb.storages import MemoryStorage
from tinydb_serialization import Serializer
import NuRadioReco.detector.detector
from NuRadioReco.detector.detector import DateTimeSerializer
import copy
logger = logging.getLogger('NuRadioReco.genericDetector')

serialization = SerializationMiddleware()
serialization.register_serializer(DateTimeSerializer(), 'TinyDate')


class GenericDetector(NuRadioReco.detector.detector.Detector):
    """
    Used the same way as the main detector class, but works with incomplete
    detector descriptions.
    The user can define a default station. If any property is missing from one
    of the other station, the value from the default station will be used instead.
    If no channels are specified for a station, the channels from the default station
    will be used instead.
    For cases when the station design has a lot of channels, a default channel can
    also be defined. The default channel has to be part of the default station.
    It works the same way as the default station: Any property missing from one
    of the other channels will be taken from the default channel.
    The GenericDetector also ignores commission and decommision times and should
    therefore not be used for real data, but only for simulation studies.
    This detector only accepts json detector descriptions or dictionary.
    """
    def __init__(self, json_filename, default_station, default_channel=None, source='json', dictionary=None, assume_inf=True):
        """
        Initialize the stations detector properties.

        Parameters
        ----------
        json_filename : str
            the path to the json detector description file (if first checks a path relative to this directory, then a
            path relative to the current working directory of the user)
            default value is 'ARIANNA/arianna_detector_db.json'
        default_station:
            ID of the station that should be used as the default station. The default station needs to have a complete detector
            description. If a property is missing in any of the other stations, the value from the default station will be used instead
        default_channel:
            ID of the channel that should be used as the default channel. This channel has to be part of the default station and have a
            complete detector description. If a property is missing in any of the other channels, the value from the default channel
            will be used instead.
        source: str
            'json' or 'dictionary'
            default value is 'json'
            If 'json' is passed, the JSON dictionary at the location specified
            by json_filename will be used
            If 'dictionary' is passed, the dictionary specified by the parameter
            'dictionary' will be used
        dictionary: dict
            If 'dictionary' is passed to the parameter source, the dictionary
            passed to this parameter will be used for the detector description.
        assume_inf : Bool
            Default to True, if true forces antenna madels to have infinite boundary conditions, otherwise the antenna madel will be determined by the station geometry.
        """
        super(GenericDetector, self).__init__(source, json_filename, dictionary, assume_inf)

        self.__default_station_id = default_station
        self.__default_channel_id = default_channel
        self.__station_changes_for_event = []
        self.__run_number = None
        self.__event_id = None
        if not self.has_station(self.__default_station_id):
            raise ValueError('The default station {} was not found in the detector description'.format(self.__default_station_id))

        Station = Query()
        self.__default_station = self._stations.get((Station.station_id == self.__default_station_id))

        if default_channel is not None:
            Channel = Query()
            self.__default_channel = self._channels.get((Channel.station_id == default_station)&(Channel.channel_id == default_channel))
            if self.__default_channel is None:
                raise ValueError('The default channel {} of station {} was not found in the detector description'.format(default_channel, self.__default_station_id))
        else:
            self.__default_channel = None

    def _get_station(self, station_id):
        if(station_id not in self._buffered_stations.keys()):
            self._buffer(station_id)
        res = copy.copy(self._buffered_stations[station_id])
        if self.__run_number is not None and self.__event_id is not None:
            for change in self.__station_changes_for_event:
                if change['station_id'] == station_id and change['run_number'] == self.__run_number and change['event_id'] == self.__event_id:
                    for name, value in change['properties'].items():
                        res[name] = value
        return res

    def _query_station(self, station_id, raw=False):
        Station = Query()
        res = self._stations.get((Station.station_id == station_id))
        if(res is None and not raw):
            logger.error("query for station {} returned no results".format(station_id))
            raise LookupError("query for station {} returned no results".format(station_id))
        if not raw:
            for key in self.__default_station.keys():
                if key not in res.keys():
                    #   if a property is missing, we use the value from the default station instead
                    res[key] = self.__default_station[key]
        return res

    def _query_channels(self, station_id, raw=False):
        Channel = Query()
        res = self._channels.search((Channel.station_id == station_id))
        if not raw:
            if len(res) == 0:
                default_channels = self._channels.search((Channel.station_id == self.__default_station_id))
                res = []
                for channel in default_channels:
                    new_channel = copy.copy(channel)
                    new_channel['station_id'] = station_id
                    res.append(new_channel)
            if self.__default_channel is not None:
                for channel in res:
                    for key in self.__default_channel.keys():
                        if key not in channel.keys() and key != 'station_id':
                            channel[key] = self.__default_channel[key]
        return res

    def _buffer(self, station_id):
        self._buffered_stations[station_id] = self._query_station(station_id)
        channels = self._query_channels(station_id)
        self._buffered_channels[station_id] = {}
        for channel in channels:
            self._buffered_channels[station_id][channel['channel_id']] = channel

    def add_generic_station(self, station_dict):
        """
        Add a generic station to the detector. The station is treated like a
        generic station in the original detector description file, i.e. all missing
        properties and all channels will be taken from the default station.
        If a station with the same ID already exists, this function does nothing.

        Parameters
        --------------
        station_dict: dictionary
            dictionary containing the station properties. Needs to at least include
            a station_id, any other missing parameters will be taken from the
            default station
        """
        if station_dict['station_id'] in self._buffered_stations.keys():
            logger.warning('Station with ID {} already exists in buffer. Cannot add station with same ID'.format(station_dict['station_id']))
            return

        for key in self.__default_station.keys():
            if key not in station_dict.keys():
                station_dict[key] = self.__default_station[key]
        self._buffered_stations[station_dict['station_id']] = station_dict

        if self.__default_station_id not in self._buffered_channels.keys():
            self._buffer(self.__default_station_id)
        self._buffered_channels[station_dict['station_id']] = {}

        for i_channel, channel in self._buffered_channels[self.__default_station_id].items():
            new_channel = copy.copy(channel)
            new_channel['station_id'] = station_dict['station_id']
            self._buffered_channels[station_dict['station_id']][channel['channel_id']] = new_channel

    def add_station_properties_for_event(self, properties, station_id, run_number, event_id):
        """
        Adds an entry to the list of event-specific changes to the detector
        description.

        Parameters
        ------------------
        properties: dictionary
            Dictionary of the properties that should be changed, with keys being
            any of the property names in the detector description and values the
            values that these properties should be changed to
        station_id: integer
            ID of the station whose properties should be changed
        run_number: integer
            Run number of the event for which the changes are valid
        event_id: integer
            Event ID of the event for which the changes are valid
        """
        self.__station_changes_for_event.append({
            'run_number': run_number,
            'event_id': event_id,
            'station_id': station_id,
            'properties': properties
        })

    def get_station_properties_for_event(self, run_number, event_id, station_id=None):
        """
        Returns all event-specific changes that have been stored in the
        detector description for a given station and event

        Parameters
        ----------------------
        run_number: integer
            Run number of the event for which the changes should be returned
        event_id: integer
            Event ID of the event for which the changes should be returned
        station_id: integer or None
            ID of the station for which the changes should be returned
            If station_id is None, changes for all stations are returned
        """
        changes = []
        for change in self.__station_changes_for_event:
            if (change['run_number'] == run_number and
            change['event_id'] == event_id):
                if station_id is None or change['station_id'] == station_id:
                    changes.append(change)
        return changes

    def set_event(self,run_number, event_id):
        """
        Sets the run number and event ID for which the detector description
        should be returned. This is needed if event-specific changes to the
        detector description have been stored. If run_number or event_id are
        not set (or are set to None), event-specific changes to the detector
        will be ignored

        Parameters
        ------------------
        run_number: integer
            Run number of the event the detector should be set to
        event_id: integer
            ID of the event the detector should be set to
        """
        self.__run_number = run_number
        self.__event_id = event_id

    def get_default_station(self):
        """
        Get the properties of the default station
        """
        return self.__default_station

    def get_default_station_id(self):
        """
        Get the ID of the default station
        """
        return self.__default_station_id

    def get_default_channel(self):
        """
        Get the properties of the default channel
        """
        return self.__default_channel

    def get_default_channel_id(self):
        """
        Get the ID of the default channel
        """
        return self.__default_channel_id

    def get_raw_station(self, station_id):
        """
        Get the properties of a station as they are in the original detector
        description, i.e. without missing properties being replaced by those
        from the default station. Event-specific changes are also ignored.

        Parameters
        --------------------------
        station_id: integer
            ID of the requested station
        """
        if station_id in self._buffered_stations.keys():
            return self._buffered_stations[station_id]
        station = self._query_station(station_id, True)
        if station is None:
            return {
                "station_id": station_id
            }
        else:
            return station

    def get_raw_channel(self, station_id, channel_id):
        """
        Get the properties of a channel as they are in the original detector
        description, i.e. without missing properties being replaced by those
        from the default channel.

        Parameters
        --------------------------
        station_id: integer
            ID of the requested channel's station
        channel_id: integer
            ID of the requested channel
        """

        if station_id in self._buffered_channels.keys():
            if channel_id in self._buffered_channels[station_id].keys():
                return self._buffered_channels[station_id][channel_id]
        channels = self._query_channels(station_id, True)
        for channel in channels:
            if channel['channel_id'] == channel_id:
                return channel
        return None

    def has_station(self, station_id):
        if station_id in self._buffered_stations.keys():
            return True
        Station = Query()
        res = self._stations.get(Station.station_id == station_id)
        return res != None

    # overwrite update function to do nothing
    def update(self, time):
        return



import numpy as np
from numpy.polynomial import polynomial as pol
import copy
import matplotlib.pyplot as plt
import logging
import datetime
import os

from radiotools import plthelpers as php

from NuRadioReco.utilities import units
import NuRadioReco.framework.channel
import NuRadioReco.framework.station
import NuRadioReco.framework.event
import NuRadioReco.modules.channelBandPassFilter
import NuRadioReco.modules.channelGenericNoiseAdder
import NuRadioReco.modules.efieldToVoltageConverter
import NuRadioReco.modules.ARIANNA.hardwareResponseIncorporator
import NuRadioReco.modules.channelGenericNoiseAdder

import NuRadioReco.modules.ARA.triggerSimulator
import NuRadioReco.modules.trigger.highLowThreshold
from NuRadioMC.SignalGen import parametrizations as signalgen

from NuRadioReco.framework.parameters import stationParameters as stnp
from NuRadioReco.framework.parameters import channelParameters as chp
from NuRadioReco.framework.parameters import electricFieldParameters as efp

from NuRadioReco.detector import detector

from NuRadioReco.modules.base import module
logger = module.setup_logger(level=logging.DEBUG)

#fig = plt.gcf()
#fig.canvas.manager.window.tkraise()


def get_ARA_power_mean_rms(sampling_rate, Vrms, min_freq, max_freq):
    """
    helper function to calculate the mean and rms power of the ARA tunnel diode
    for a given Vrms, sampling rate and frequency content

    Parameters
    ----------
    sampling_rate: float
        the sampling rate e.g. 1GHz
    Vrms: float
        the RMS of noise in the time domain
    min_freq: float
        the lower bandpass frequency
    max_freq: float
        the upper bandpass frequency
    """
    triggerSimulator = NuRadioReco.modules.ARA.triggerSimulator.triggerSimulator()
    channelGenericNoiseAdder = NuRadioReco.modules.channelGenericNoiseAdder.channelGenericNoiseAdder()

    noise = NuRadioReco.framework.channel.Channel(0)

    long_noise = channelGenericNoiseAdder.bandlimited_noise(min_freq=min_freq,
                                                            max_freq=max_freq,
                                                            n_samples=2 ** 20,
                                                            sampling_rate=sampling_rate,
                                                            amplitude=Vrms,
                                                            type='perfect_white')
    long_noise *= Vrms / long_noise.std()

    print(long_noise.std())

    noise.set_trace(long_noise, sampling_rate)

    power_noise = triggerSimulator.tunnel_diode(noise)

    power_mean = np.mean(power_noise)
    power_rms = np.sqrt(np.mean(power_noise ** 2))
    return power_mean, power_rms


det = detector.Detector(json_filename='../example_data/dummy_detector.json')
det.update(datetime.datetime(2018, 10, 1))

efieldToVoltageConverter = NuRadioReco.modules.efieldToVoltageConverter.efieldToVoltageConverter()
efieldToVoltageConverter.begin()
channelBandPassFilter = NuRadioReco.modules.channelBandPassFilter.channelBandPassFilter()
hardwareResponseIncorporator = NuRadioReco.modules.ARIANNA.hardwareResponseIncorporator.hardwareResponseIncorporator()
channelGenericNoiseAdder = NuRadioReco.modules.channelGenericNoiseAdder.channelGenericNoiseAdder()

logger.setLevel(logging.WARNING)


n_index = 1.78
cherenkov_angle = np.arccos(1. / n_index)
dt = 1. * units.ns
N = 2 ** 8

Vrms = 11 * units.micro * units.V

NN = 1000
min_freq, max_freq = 100 * units.MHz, .5 * units.GHz
if 0:
    # now calculate the relation between the ARA SNR (integrated power ratio) and the ARIANNA SNR (Vp2p/2/Vrms)
    import NuRadioReco.modules.ARA.triggerSimulator
    triggerSimulator = NuRadioReco.modules.ARA.triggerSimulator.triggerSimulator()
    power_mean, power_rms = get_ARA_power_mean_rms(1. / dt, Vrms, min_freq, max_freq)
    print("power mean/rms = {:.4g}/{:.4g}".format(power_mean, power_rms))
    counter = -1
    SNRp2p_bicone = np.zeros(NN)
    SNRara_bicone = np.zeros(NN)
    for E in 10 ** np.linspace(15.5, 17, 100):
        for theta in np.linspace(cherenkov_angle - 0.1 * units.deg, cherenkov_angle + 0.1 * units.deg, 10):
            counter += 1
            pulse = signalgen.get_time_trace(E, theta, N, dt, 'HAD', n_index, 1000 * units.m, 'Alvarez2000')
            event = NuRadioReco.framework.event.Event(1, 1)
            station = NuRadioReco.framework.station.Station(101)
            trace = np.zeros((3, N))
            trace[1] = pulse
            trace[2] = pulse
            sim_station = NuRadioReco.framework.sim_station.SimStation(101)
            electric_field = NuRadioReco.framework.electric_field.ElectricField([2])
            electric_field.set_trace(sampling_rate=1. / dt, trace=trace)
            electric_field[efp.azimuth] = 0
            electric_field[efp.zenith] = (90 + 45) * units.deg
            electric_field[efp.ray_path_type] = 'direct'
            sim_station.add_electric_field(electric_field)
            sim_station[stnp.zenith] = (90 + 45) * units.deg
            sim_station[stnp.zenith] = 0
            station.set_sim_station(sim_station)
            event.set_station(station)

            efieldToVoltageConverter.run(event, station, det)
            channelBandPassFilter.run(event, station, det, passband=[min_freq, max_freq])

            trace_bicone = station.get_channel(2).get_trace()

            SNRp2p_bicone[counter] = (trace_bicone.max() - trace_bicone.min()) / 2. / Vrms

            after_tunnel_diode = np.abs(triggerSimulator.tunnel_diode(station.get_channel(2)))
            power_mean = 0
            SNRara_bicone[counter] = np.max((after_tunnel_diode - power_mean) / power_rms)

    fig, ax = plt.subplots(1, 1)
    ax.scatter(SNRp2p_bicone, SNRara_bicone, s=20, alpha=0.5)
    ax.set_xlabel("ARIANNA SNR (Vp2p/2/Vrms)")
    ax.set_ylabel("ARA SNR (tunnel diode output/noise power RMS)")
    ax.set_title("for ARA bicone response")
    fig.tight_layout()
    fig.savefig('plots/SNR_ARA_ARIANNA.png')
    plt.show()

long_noise = channelGenericNoiseAdder.bandlimited_noise(min_freq=min_freq,
                                            max_freq=max_freq,
                                            n_samples=2 ** 20,
                                            sampling_rate=1 / dt,
                                            amplitude=Vrms,
                                            type='perfect_white')
NN = 10000
SS_LPDA = np.zeros(NN)
SS_LPDA_amp = np.zeros(NN)
Vp2p_LPDA = np.zeros(NN)
Vp2p_LPDA_noise_1 = np.zeros(NN)
Vp2p_LPDA_noise_2 = np.zeros(NN)
Vp2p_LPDA_amp = np.zeros(NN)
SS_bicone = np.zeros(NN)
Vp2p_bicone = np.zeros(NN)
Vp2p_bicone_noise_1 = np.zeros(NN)
Vp2p_bicone_noise_2 = np.zeros(NN)
counter = -1
if not os.path.isdir('plots'):
    os.mkdir('plots')
for signal_scaling in np.linspace(1, 100, NN):
    for theta in np.linspace(cherenkov_angle - 3 * units.deg, cherenkov_angle * units.deg, 1):
        counter += 1
        print(counter)
        pulse = signalgen.get_time_trace(1e18, theta, N, dt, 'HAD', n_index, 1000 * units.m, 'Alvarez2000')
        event = NuRadioReco.framework.event.Event(1, 1)
        station = NuRadioReco.framework.station.Station(101)
        trace = np.zeros((3, N))
        trace[1] = pulse
        trace[2] = pulse
        trace = trace / np.max(np.abs(trace)) * signal_scaling * Vrms
        sim_station = NuRadioReco.framework.sim_station.SimStation(101)
        electric_field = NuRadioReco.framework.electric_field.ElectricField([2,3,4,5])
        electric_field.set_trace(sampling_rate=1. / dt, trace=trace)
        electric_field[efp.azimuth] = 0
        electric_field[efp.zenith] = (90 + 45) * units.deg
        electric_field[efp.ray_path_type] = 'direct'
        electric_field.set_trace_start_time(0)
        electric_field[efp.zenith] = (90 + 45) * units.deg
        electric_field[efp.azimuth] = 0
        sim_station.add_electric_field(electric_field)
        station.set_sim_station(sim_station)
        event.set_station(station)

        efieldToVoltageConverter.run(event, station, det)
        channelBandPassFilter.run(event, station, det, passband=[80 * units.MHz, 500 * units.MHz])

        trace_LPDA = copy.copy(station.get_channel(3).get_trace())
        trace_bicone = station.get_channel(2).get_trace()

        def get_Vp2p(trace, dt=dt, coincidence_window=5 * units.ns):
            tstart = 20 * units.ns
            tstop = 50 * units.ns
            maximum = 0
            for t in np.arange(tstart, tstop, dt):
                n_bins = np.int(np.ceil(coincidence_window / dt))
                bin_start = np.int(np.ceil(t / dt))
                maximum = max(maximum, np.max(trace[bin_start:(bin_start+n_bins)]) - np.min(trace[bin_start:(bin_start+n_bins)]))
            return maximum
#             return np.max(trace) - np.min(trace)
#             index = np.argmax(np.abs(trace))
#             n_bins = np.int(np.ceil(coincidence_window / dt))
#             maximum = trace[index]
#             sign = np.sign(maximum)
#             minimum = np.max(-1 * sign * trace[max(0, (index - n_bins)):min((index + n_bins), len(trace))])
#             return np.abs(maximum) + np.abs(minimum)

        Vp2p_LPDA[counter] = get_Vp2p(trace_LPDA)
        Vp2p_bicone[counter] = get_Vp2p(trace_bicone)

        SS_LPDA[counter] = np.sum(trace_LPDA ** 2) * dt
        SS_bicone[counter] = np.sum(trace_bicone ** 2) * dt

        if(counter < 0):  # plot some example traces
            fig, ax = plt.subplots(1, 3, sharey=True, figsize=(20, 7))
            ax = np.array(ax).flatten()
            tt = station.get_channel(3).get_times()
            ax[0].plot(tt / units.ns, pulse / units.micro / units.V)
            ax[1].plot(tt / units.ns, trace_bicone / units.micro / units.V)
            ax[2].plot(tt / units.ns, trace_LPDA / units.micro / units.V)
            ax[0].set_ylabel('voltage [$\mu$V]')
            ax[2].set_ylabel('voltage [mV]')
            ax[0].set_title("no antenna")
            ax[1].set_title("bicone")
            ax[2].set_title("LPDA")
            ax[0].set_xlabel("time [ns]")
            ax[1].set_xlabel("time [ns]")
            ax[2].set_xlabel("time [ns]")
            fig.tight_layout()
            fig.suptitle(r"$\theta$ = {:.0f}, dCherenkov = {:.0f}".format(135, (theta - cherenkov_angle) / units.deg))
            fig.subplots_adjust(top=0.9)
            fig.savefig("plots/trace_LPDA_bicone_{:d}.png".format(counter))
            plt.show()
            plt.close("all")

        channelGenericNoiseAdder.run(event, station, det, amplitude=Vrms,
                                     min_freq=100 * units.MHz,
                                     max_freq=500 * units.MHz,
                                     type='perfect_white')
        trace_LPDA = station.get_channel(3).get_trace()
        trace_bicone = station.get_channel(2).get_trace()
        Vp2p_LPDA_noise_1[counter] = get_Vp2p(trace_LPDA)
        Vp2p_bicone_noise_1[counter] = get_Vp2p(trace_bicone)

        trace_LPDA = station.get_channel(5).get_trace()
        trace_bicone = station.get_channel(4).get_trace()
        Vp2p_LPDA_noise_2[counter] = get_Vp2p(trace_LPDA)
        Vp2p_bicone_noise_2[counter] = get_Vp2p(trace_bicone)

        hardwareResponseIncorporator.run(event, station, det, sim_to_data=True)
        trace_LPDA_amp = station.get_channel(3).get_trace()
        Vp2p_LPDA_amp[counter] = get_Vp2p(trace_LPDA_amp)
        SS_LPDA_amp[counter] = np.sum(trace_LPDA_amp ** 2) * dt


fig, (ax, ax2) = plt.subplots(1, 2)
ax.scatter(Vp2p_LPDA / Vrms / 2, SS_LPDA / Vrms ** 2, label='LPDA')
ax.scatter(Vp2p_bicone / Vrms / 2, SS_bicone / Vrms ** 2, label='bicone')
ax.set_ylabel(r"$\sum A_i^2 \Delta t / V_\mathrm{RMS}^2 [ns]$")
ax.set_xlabel(r"$V_\mathrm{p2p}/2/V_\mathrm{RMS}$")
ax.semilogx(True)
ax.semilogy(True)
ax2.set_ylabel(r"$\sqrt{\sum A_i^2 \Delta t/ V_\mathrm{RMS}^2 [ns]}$")
ax2.set_xlabel(r"$V_\mathrm{p2p}/2/V_\mathrm{RMS}$")
ax2.scatter(Vp2p_LPDA / Vrms / 2, (SS_LPDA / units.ns / Vrms ** 2) ** 0.5, label='LPDA', s=15)
ax2.scatter(Vp2p_bicone / Vrms / 2, (SS_bicone / units.ns / Vrms ** 2) ** 0.5, label='bicone', s=15)
coef_bicone = pol.polyfit(Vp2p_bicone / Vrms / 2, (SS_bicone / units.ns / Vrms ** 2) ** 0.5, deg=[1])
coef_LPDA = pol.polyfit(Vp2p_LPDA / Vrms / 2, (SS_LPDA / units.ns / Vrms ** 2) ** 0.5, deg=[1])
print("slope LPDA: SNR_new = ({:.5f} * SNR_p2p + {:.5f} ".format(coef_LPDA[1], coef_LPDA[0]))
print("slope bicone: SNR_new = ({:.5f} * SNR_p2p + {:.5f} ".format(coef_bicone[1], coef_bicone[0]))
xx = np.linspace(0, 150)
ax2.plot(xx, pol.polyval(xx, coef_bicone), '--C3')
ax2.plot(xx, pol.polyval(xx, coef_LPDA), '--C2')
ax2.text(5, 7, "LPDA: SNR_new = ({:.2f} * SNR_p2p)^2".format(coef_LPDA[1]), fontsize='x-small')
ax2.text(5, 4, "bicone: SNR_new = ({:.2f} * SNR_p2p)^2".format(coef_bicone[1]), fontsize='x-small')
ax.legend()
ax2.set_ylim(0, 250 / 10)
ax2.set_xlim(0, 150 / 10)
ax2.legend()
fig.tight_layout()
fig.savefig("plots/SNRcomparison.png".format(counter))
# plt.show()

# calculate trigger efficiency of ARIANNA Vp2p trigger
bins = np.arange(0, 150, 5)
bins_center = 0.5 * (bins[1:] + bins[:-1])

SNR_bicone = SS_bicone / Vrms ** 2 / units.ns
SNR_ARIANNA_bicone = Vp2p_bicone / Vrms / 2.
weights = (SNR_ARIANNA_bicone >= 3) * np.ones(len(SNR_ARIANNA_bicone))
Hcount, edges = np.histogram(SNR_bicone, bins=bins)
Hweight, edges = np.histogram(SNR_bicone, bins=bins, weights=weights)
trigger_efficiency_bicone = Hweight / Hcount

SNR_ARIANNA_LPDA = Vp2p_LPDA / Vrms / 2.
SNR_LPDA = SS_LPDA / Vrms ** 2 / units.ns
weights = (SNR_ARIANNA_LPDA >= 3) * np.ones(len(SNR_ARIANNA_LPDA))
Hcount, edges = np.histogram(SNR_LPDA, bins=bins)
Hweight, edges = np.histogram(SNR_LPDA, bins=bins, weights=weights)
trigger_efficiency_LPDA = Hweight / Hcount

Vrms_amp = (SS_LPDA_amp / SS_LPDA).mean() ** 0.5 * Vrms  # quick and dirty hack to get the Vrms after the amp response
SNR_ARIANNA_LPDA_amp = Vp2p_LPDA_amp / Vrms_amp / 2.
SNR_LPDA_amp = SS_LPDA_amp / Vrms_amp ** 2 / units.ns
weights = (SNR_ARIANNA_LPDA_amp >= 3) * np.ones(len(SNR_ARIANNA_LPDA_amp))
Hcount, edges = np.histogram(SNR_LPDA_amp, bins=bins)
Hweight, edges = np.histogram(SNR_LPDA_amp, bins=bins, weights=weights)
trigger_efficiency_LPDA_amp = Hweight / Hcount

fig, ax = plt.subplots(1, 1)
ax.plot(bins_center, trigger_efficiency_bicone, 'o-', label='bicone')
# ax.plot(bins_center, trigger_efficiency_LPDA, 's-', label='LPDA')
# ax.plot(bins_center, trigger_efficiency_LPDA_amp, 'd--', label='LPDA + amp')
ax.set_xticks(np.arange(0, 180, 20))
ax.set_xlim(0, 140)
ax.set_xlabel("new SNR")
ax.set_ylabel("trigger efficiency")
ax.set_title("trigger settings: Vp2p/Vrms/2 > 3, (3/8 trigger rate ~10mHz)")
ax.legend()
fig.tight_layout()
fig.savefig("plots/ARIANNAtriggerefficiency_allcangles.png".format(counter))

# with noise
SNR_ARIANNA_bicone_1 = Vp2p_bicone_noise_1 / Vrms / 2.
SNR_ARIANNA_bicone_2 = Vp2p_bicone_noise_2 / Vrms / 2.
weights_1 = (SNR_ARIANNA_bicone_1 >= 3) * np.ones(len(SNR_ARIANNA_bicone_1))
weights_2 = (SNR_ARIANNA_bicone_2 >= 3) * np.ones(len(SNR_ARIANNA_bicone_2))
weights = weights_1 * weights_2
Hcount, edges = np.histogram(SNR_bicone, bins=bins)
Hweight, edges = np.histogram(SNR_bicone, bins=bins, weights=weights)
trigger_efficiency_bicone = Hweight / Hcount
mask = (SNR_ARIANNA_bicone_1 >= 3) & (SNR_ARIANNA_bicone_2 >= 3)
bis = np.arange(0, 150, 5)
fig, ax = php.get_histograms([SNR_bicone, SNR_bicone[mask]], xlabels=['SNR all', 'SNR triggered'],
                             bins=[bins, bins])
fig.savefig('plots/ARIANNAtriggereffSNR_5ns.png')

weights_1 = (SNR_ARIANNA_bicone_1 >= 4) * np.ones(len(SNR_ARIANNA_bicone_1))
weights_2 = (SNR_ARIANNA_bicone_2 >= 4) * np.ones(len(SNR_ARIANNA_bicone_2))
weights = weights_1 * weights_2
Hcount, edges = np.histogram(SNR_bicone, bins=bins)
Hweight, edges = np.histogram(SNR_bicone, bins=bins, weights=weights)
trigger_efficiency_bicone_4s = Hweight / Hcount

weights_1 = (SNR_ARIANNA_bicone_1 >= 5) * np.ones(len(SNR_ARIANNA_bicone_1))
weights_2 = (SNR_ARIANNA_bicone_2 >= 5) * np.ones(len(SNR_ARIANNA_bicone_2))
weights = weights_1 * weights_2
Hcount, edges = np.histogram(SNR_bicone, bins=bins)
Hweight, edges = np.histogram(SNR_bicone, bins=bins, weights=weights)
trigger_efficiency_bicone_5s = Hweight / Hcount

SNR_ARIANNA_LPDA_1 = Vp2p_LPDA_noise_1 / Vrms / 2.
SNR_ARIANNA_LPDA_2 = Vp2p_LPDA_noise_2 / Vrms / 2.
weights_1 = (SNR_ARIANNA_LPDA_1 >= 3) * np.ones(len(SNR_ARIANNA_LPDA_1))
weights_2 = (SNR_ARIANNA_LPDA_2 >= 3) * np.ones(len(SNR_ARIANNA_LPDA_2))
weights = weights_1 * weights_2
Hcount, edges = np.histogram(SNR_LPDA, bins=bins)
Hweight, edges = np.histogram(SNR_LPDA, bins=bins, weights=weights)
trigger_efficiency_LPDA = Hweight / Hcount

weights_1 = (SNR_ARIANNA_LPDA_1 >= 4) * np.ones(len(SNR_ARIANNA_LPDA_1))
weights_2 = (SNR_ARIANNA_LPDA_2 >= 4) * np.ones(len(SNR_ARIANNA_LPDA_2))
weights = weights_1 * weights_2
Hcount, edges = np.histogram(SNR_LPDA, bins=bins)
Hweight, edges = np.histogram(SNR_LPDA, bins=bins, weights=weights)
trigger_efficiency_LPDA_4s = Hweight / Hcount

fig, ax = plt.subplots(1, 1)
ax.plot(bins_center, trigger_efficiency_bicone, 'o-', label='bicone 3sigma (with noise)')
ax.plot(bins_center, trigger_efficiency_bicone_4s, 'o-', label='bicone 4sigma (with noise)')
ax.plot(bins_center, trigger_efficiency_bicone_5s, 'o-', label='bicone 5sigma (with noise)')
# ax.plot(bins_center, trigger_efficiency_LPDA, 's--', label='LPDA 3sigma (with noise)')
# ax.plot(bins_center, trigger_efficiency_LPDA_4s, 's--', label='LPDA 4sigma (with noise)')
# ax.plot(bins_center, trigger_efficiency_LPDA_amp, 'd--', label='LPDA + amp')
ax.set_xticks(np.arange(0, 180, 20))
ax.set_xlim(0, 140)
ax.set_xlabel("new SNR")
ax.set_ylabel("trigger efficiency")
ax.set_title("trigger settings: Vp2p/Vrms/2 > N, 2/2 trigger")
ax.legend()
fig.tight_layout()
fig.savefig("plots/ARIANNAtriggerefficiency_allcangles_noise_5ns.png".format(counter))
plt.show()



import dash
import json
import plotly
import numpy as np
from app import app
import dash_html_components as html
import dash_core_components as dcc
from dash.dependencies import Input, Output, State
from NuRadioReco.utilities import units
from NuRadioReco.framework.parameters import stationParameters as stnp
import NuRadioReco.eventbrowser.apps.common
import dataprovider
provider = dataprovider.DataProvider()

layout = [
    dcc.RadioItems(
        id='station-overview-rec-sim',
        options=[
            {'label': 'Reconstruction', 'value': 'rec'},
            {'label': 'Simulation', 'value': 'sim'}
        ],
        value='rec'
    ),
    html.Div(id='station-overview-properties')
]

station_properties_for_overview = [
    {
        'label': 'Zenith [deg]',
        'param': stnp.zenith,
        'unit': units.deg
    },{
        'label': 'Azimuth [deg]',
        'param': stnp.azimuth,
        'unit': units.deg
    },{
        'label': 'Neutrino Energy [eV]',
        'param': stnp.nu_energy,
        'unit': units.eV
    },{
        'label': 'Cosmic Ray Energy [eV]',
        'param': stnp.cr_energy,
        'unit': units.eV
    }
]
@app.callback(Output('station-overview-properties', 'children'),
                [Input('filename', 'value'),
                Input('event-counter-slider', 'value'),
                Input('station-id-dropdown', 'value'),
                Input('station-overview-rec-sim', 'value')],
                [State('user_id', 'children')])
def station_overview_properties(filename, evt_counter, station_id, rec_or_sim, juser_id):
    if filename is None or station_id is None:
        return ''
    user_id = json.loads(juser_id)
    ariio = provider.get_arianna_io(user_id, filename)
    evt = ariio.get_event_i(evt_counter)
    station = evt.get_station(station_id)
    if station is None:
        return []
    if rec_or_sim == 'rec':
        prop_station = station
    else:
        if station.get_sim_station() is None:
            return []
        else:
            prop_station = station.get_sim_station()
    if prop_station.is_neutrino():
        event_type = 'Neutrino'
    elif prop_station.is_cosmic_ray():
        event_type = 'Cosmic Ray'
    else:
        event_type = 'Unknown'
    reply = []
    reply.append(
        html.Div([
            html.Div('Event Type:', className='custom-table-td'),
            html.Div(str(event_type), className='custom-table-td custom-table-td-last')
        ], className='custom-table-row')
    )
    props = NuRadioReco.eventbrowser.apps.common.get_properties_divs(prop_station, station_properties_for_overview)
    for prop in props:
        reply.append(prop)
    return reply



import numpy as np
from NuRadioReco.modules.base.module import register_run
from NuRadioReco.utilities import geometryUtilities as geo_utl
from NuRadioReco.utilities import units
from NuRadioReco.utilities import ice
from NuRadioReco.framework.parameters import channelParameters as chp
from NuRadioReco.framework.parameters import electricFieldParameters as efp
# from detector import antennamodel
from NuRadioReco.detector import antennapattern
from radiotools import coordinatesystems
import copy
import time
import logging
import fractions
from scipy import signal
from decimal import Decimal
import NuRadioReco.framework.channel
from NuRadioReco.utilities import fft
logger = logging.getLogger('calculateAmplitudePerRaySolution')


class calculateAmplitudePerRaySolution:
    """
    Convolves the signal from each ray tracing solution with the antenna response
    and calculates the maximum signal amplitude
    """

    def __init__(self):
        self.__t = 0
        self.begin()

    def begin(self, debug=False):
        """
        begin method, sets general parameters of module

        """
        self.__debug = debug
        self.antenna_provider = antennapattern.AntennaPatternProvider()

    @register_run()
    def run(self, evt, station, det):
        t = time.time()

        # access simulated efield and high level parameters
        sim_station = station.get_sim_station()
        sim_station_id = sim_station.get_id()

        # loop over all channels
        for efield in sim_station.get_electric_fields():
            # one efield might be valid for multiple channels, hence we loop over all channels this efiels is valid for,
            # convolve each trace with the antenna response for the given angles
            # and transform it to the time domain to calculate the max. amplitude
            channel_ids = efield.get_channel_ids()
            for channel_id in channel_ids:
                logger.debug('channel id {}'.format(channel_id))

                zenith = efield[efp.zenith]
                azimuth = efield[efp.azimuth]

                ff = efield.get_frequencies()
                efield_fft = efield.get_frequency_spectrum()

                # get antenna pattern for current channel
                antenna_model = det.get_antenna_model(sim_station_id, channel_id, zenith)
                antenna_pattern = self.antenna_provider.load_antenna_pattern(antenna_model, interpolation_method='complex')
                ori = det.get_antenna_orientation(sim_station_id, channel_id)
                logger.debug("zen {:.0f}, az {:.0f}".format(zenith / units.deg, azimuth / units.deg))
                VEL = antenna_pattern.get_antenna_response_vectorized(ff, zenith, azimuth, *ori)

                # Apply antenna response to electric field
                voltage_fft = efield_fft[2] * VEL['phi'] + efield_fft[1] * VEL['theta']

                # Remove DC offset
                voltage_fft[np.where(ff < 5 * units.MHz)] = 0.

                voltage = fft.freq2time(voltage_fft, efield.get_sampling_rate())
                h = np.abs(signal.hilbert(voltage))
                maximum = np.abs(voltage).max()
                maximum_envelope = h.max()

                if not efield.has_parameter(efp.max_amp_antenna):
                    efield[efp.max_amp_antenna] = {}
                    efield[efp.max_amp_antenna_envelope] = {}
                efield[efp.max_amp_antenna][channel_id] = maximum
                efield[efp.max_amp_antenna_envelope][channel_id] = maximum_envelope

        self.__t += time.time() - t

    def end(self):
        from datetime import timedelta
        logger.setLevel(logging.INFO)
        dt = timedelta(seconds=self.__t)
        logger.info("total time used by this module is {}".format(dt))
        return dt



from __future__ import print_function
from NuRadioReco.modules.base.module import register_run
import numpy as np
from NuRadioReco.utilities import units, fft
import logging



class channelGenericNoiseAdder:
    """
    Module that generates noise in some generic fashion (not based on measured data), which can be added to data.


    """

    def add_random_phases(self, amps, n_samples_time_domain):
        """
        Adding random phase information to given amplitude spectrum.

        Parameters
        ---------

        amps: array of floats
            Data that random phase is added to.
        n_samples_time_domain: int
            number of samples in the time domain to differentiate between odd and even number of samples
        """
        amps = np.array(amps, dtype='complex')
        Np = (n_samples_time_domain - 1) // 2
        phases = np.random.rand(Np) * 2 * np.pi
        phases = np.cos(phases) + 1j * np.sin(phases)
        amps[1:Np + 1] *= phases  # Note that the last entry of the index slice is f[Np] !

        return amps

    def fftnoise_fullfft(self, f):
        """
        Adding random phase information to given amplitude spectrum.

        Parameters
        ---------

        f: array of floats
            Data that random phase is added to.
        """
        f = np.array(f, dtype='complex')
        Np = (len(f) - 1) // 2
        phases = np.random.rand(Np) * 2 * np.pi
        phases = np.cos(phases) + 1j * np.sin(phases)
        f[1:Np + 1] *= phases  # Note that the last entry of the index slice is f[Np] !
        f[-1:-1 - Np:-1] = np.conj(f[1:Np + 1])

        self.logger.debug(' fftnoise: Length of frequency array = {} '.format(len(f)))
        self.logger.debug(' fftnoise: Number of points for unilateral spectrum = {} '.format(Np))
        self.logger.debug(' fftnoise: Max index and amplitude of positive part of spectrum: index = {}, A = |{}| = {} '.format(Np, f[Np], abs(f[Np])))
        self.logger.debug(' fftnoise: Min index and amplitude of negative part of spectrum: index = {}, A = |{}| '.format(len(f) - Np, f[-Np]))

        fftprec = max(abs(np.fft.ifft(f) - np.fft.ifft(f).real))
        fftcheck = fftprec - np.finfo(float).resolution
        self.logger.debug(' fftnoise: fft precision {} < {} (float resolution) is : {} !'.format(fftprec , np.finfo(float).resolution, fftcheck < 0))

        if fftcheck >= 0:
            self.logger.warning(' fftnoise: Non negligibe imagniary part of inverse FFT: {} '.format(fftcheck))

        return np.fft.ifft(f).real

    def bandlimited_noise(self, min_freq, max_freq, n_samples, sampling_rate, amplitude, type='perfect_white',
                          time_domain=True, bandwidth=None):
        """
        Generating noise of n_samples in a bandwidth [min_freq,max_freq].

        Parameters
        ---------

        min_freq: float
            Minimum frequency of passband for noise generation
            min_freq = None: Only the DC component is removed. If the DC component should be included,
            min_freq = 0 has to be specified
        max_freq: float
            Maximum frequency of passband for noise generation
            If the maximum frequency is above the Nquist frequencey (0.5 * sampling rate), the Nquist frequency is used
            max_freq = None: Frequencies up to Nyquist freq are used.
        n_samples: int
            number of samples in the time domain
        sampling_rate: float
            desired sampling rate of data
        amplitude: float
            desired voltage of noise as V_rms (only roughly, since bandpass limited)
        type: string
            perfect_white: flat frequency spectrum
            rayleigh: Amplitude of each frequency bin is drawn from a Rayleigh distribution
            # white: flat frequency spectrum with random jitter
        time_domain: bool (default True)
            if True returns noise in the time domain, if False it returns the noise in the frequency domain. The latter
            might be more performant as the noise is generated internally in the frequency domain.
        bandwidth: float or None (default)
            if this parameter is specified, the amplitude is interpreted as the amplitude for the bandwidth specified here
            Otherwise the amplitude is interpreted for the bandwidth of min(max_freq, 0.5 * sampling rate) - min_freq
            If `bandwidth` is larger then (min(max_freq, 0.5 * sampling rate) - min_freq) it has the same effect as `None`

        Comments
        --------
        *   Note that by design the max frequency is the Nyquist frequency, even if a bigger max_freq
            is implemented (RL 17-Sept-2018)

        *   Add 'multi_white' noise option on 20-Sept-2018 (RL)

        """
        frequencies = np.fft.rfftfreq(n_samples, 1. / sampling_rate)

        n_samples_freq = len(frequencies)

        if min_freq == None or min_freq == 0:
            # remove DC component; fftfreq returns the DC component as 0-th element and the negative
            # frequencies at the end, so frequencies[1] should be the lowest frequency; it seems safer,
            # to take the difference between two frequencies to determine the minimum frequency, in case
            # future versions of numpy change the order and maybe put the negative frequencies first
            min_freq = 0.5 * (frequencies[2] - frequencies[1])
            self.logger.info(' Set min_freq from None to {} MHz!'.format(min_freq / units.MHz))
        if max_freq == None:
            # sample up to Nyquist frequency
            max_freq = max(frequencies)
            self.logger.info(' Set max_freq from None to {} GHz!'.format(max_freq / units.GHz))
        selection = (frequencies >= min_freq) & (frequencies <= max_freq)

        nbinsactive = np.sum(selection)
        self.logger.debug('Total number of frequency bins (bilateral spectrum) : {} , of those active: {} '.format(n_samples, nbinsactive))

        # Debug plots
#         f1 = plt.figure()
#         plt.plot (frequencies/max(frequencies))
#         plt.plot(fbinsactive,'kx')

        if(bandwidth is not None):
            sampling_bandwidth = min(0.5 * sampling_rate, max_freq) - min_freq
            amplitude *= 1. / (bandwidth / (sampling_bandwidth)) ** 0.5  # normalize noise level to the bandwidth its generated for

        ampl = np.zeros(n_samples_freq)
        sigscale = (1.*n_samples) / np.sqrt(nbinsactive)
        if type == 'perfect_white':
            ampl[selection] = amplitude * sigscale
        elif type == 'rayleigh':
            fsigma = amplitude * sigscale / np.sqrt(2.)
            ampl[selection] = np.random.rayleigh(fsigma, nbinsactive)
#         elif type == 'white':
# FIXME: amplitude normalization is not correct for 'white'
#             ampl = np.random.rand(n_samples) * 0.05 * amplitude + amplitude * np.sqrt(2.*n_samples * 2)
        else:
            self.logger.error("Other types of noise not yet implemented.")
            raise NotImplementedError("Other types of noise not yet implemented.")

        noise = self.add_random_phases(ampl, n_samples) / sampling_rate
        if(time_domain):
            return fft.freq2time(noise, sampling_rate, n=n_samples)
        else:
            return noise

    def __init__(self):
        self.begin()
        self.logger = logging.getLogger('NuRadioReco.channelGenericNoiseAdder')

    def begin(self, debug=False):
        self.__debug = debug
        if debug:
            self.logger.setLevel(logging.DEBUG)

    @register_run()
    def run(self, event, station, detector,
                            amplitude=1 * units.mV,
                            min_freq=50 * units.MHz,
                            max_freq=2000 * units.MHz,
                            type='perfect_white',
                            excluded_channels=[],
                            bandwidth=None):

        """
        Add noise to given event.

        Parameters
        ---------

        event

        station

        detector

        amplitude: float
            desired voltage of noise as V_rms for the specified bandwidth
        min_freq: float
            Minimum frequency of passband for noise generation
        max_freq: float
            Maximum frequency of passband for noise generation
            If the maximum frequency is above the Nquist frequencey (0.5 * sampling rate), the Nquist frequency is used
        type: string
            perfect_white: flat frequency spectrum
            rayleigh: Amplitude of each frequency bin is drawn from a Rayleigh distribution
        excluded_channels: list of ints
            the channels ids of channels where no noise will be added, default is that no channel is excluded
        bandwidth: float or None (default)
            if this parameter is specified, the amplitude is interpreted as the amplitude for the bandwidth specified here
            Otherwise the amplitude is interpreted for the bandwidth of min(max_freq, 0.5 * sampling rate) - min_freq
            If `bandwidth` is larger then (min(max_freq, 0.5 * sampling rate) - min_freq) it has the same effect as `None`

        """

        channels = station.iter_channels()
        for channel in channels:
            if(channel.get_id() in excluded_channels):
                continue

            trace = channel.get_trace()
            sampling_rate = channel.get_sampling_rate()

            noise = self.bandlimited_noise(min_freq=min_freq,
                                          max_freq=max_freq,
                                          n_samples=trace.shape[0],
                                          sampling_rate=sampling_rate,
                                          amplitude=amplitude,
                                          type=type,
                                          bandwidth=bandwidth)

            if self.__debug:
                new_trace = trace + noise

                self.logger.debug("imput amplitude {}".format(amplitude))
                self.logger.debug("voltage RMS {}".format(np.sqrt(np.mean(noise ** 2))))

                import matplotlib.pyplot as plt
                plt.plot(trace)
                plt.plot(noise)
                plt.plot(new_trace)

                plt.figure()
                plt.plot(np.abs(fft.time2freq(trace, channel.get_sampling_rate())))
                plt.plot(np.abs(fft.time2freq(noise, channel.get_sampling_rate())))
                plt.plot(np.abs(fft.time2freq(new_trace, channel.get_sampling_rate())))

                plt.show()

            new_trace = trace + noise
            channel.set_trace(new_trace, sampling_rate)

    def end(self):
        pass
